@@ -25,6 +25,7 @@
 #include "PdfFontMetrics.h"
 #include "PdfLocale.h"
 #include "util/PdfMutexWrapper.h"
+#include "PdfDefinesPrivate.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -50,30 +51,31 @@ PdfEncoding::~PdfEncoding()
 // PdfSimpleEncoding
 // -----------------------------------------------------
 PdfSimpleEncoding::PdfSimpleEncoding( const PdfName & rName )
-    : PdfEncoding( 0, 255 ), m_name( rName ), m_pEncodingTable( NULL )
+    : PdfEncoding( 0, 255 ), m_mutex( new PoDoFo::Util::PdfMutex() ), m_name( rName ), m_pEncodingTable( NULL )
 {
 }
 
 PdfSimpleEncoding::~PdfSimpleEncoding() 
 {
     free( m_pEncodingTable );
+    delete m_mutex;
 }
 
 void PdfSimpleEncoding::InitEncodingTable() 
 {
-    Util::PdfMutexWrapper wrapper( m_mutex );
+    Util::PdfMutexWrapper wrapper( *m_mutex );
     const long         lTableLength     = 0xffff;
     const pdf_utf16be* cpUnicodeTable   = this->GetToUnicodeTable();
 
     if( !m_pEncodingTable ) // double check
     {
-	m_pEncodingTable = static_cast<char*>(malloc(sizeof(char)*lTableLength));
+        m_pEncodingTable = static_cast<char*>(malloc(sizeof(char)*lTableLength));
     
-	// fill the table with 0
-	memset( m_pEncodingTable, 0, lTableLength * sizeof(char) ); 
-	// fill the table with data
-	for( int i=0;i<256;i++ )
-	    m_pEncodingTable[ cpUnicodeTable[i] ] = i;
+        // fill the table with 0
+        memset( m_pEncodingTable, 0, lTableLength * sizeof(char) ); 
+        // fill the table with data
+        for( int i=0;i<256;i++ )
+            m_pEncodingTable[ cpUnicodeTable[i] ] = i;
     }
 }
 
@@ -85,9 +87,9 @@ void PdfSimpleEncoding::AddToDictionary( PdfDictionary & rDictionary ) const
 pdf_utf16be PdfSimpleEncoding::GetCharCode( int nIndex ) const
 {
     if( nIndex < this->GetFirstChar() ||
-	nIndex > this->GetLastChar() )
+        nIndex > this->GetLastChar() )
     {
-	PODOFO_RAISE_ERROR( ePdfError_ValueOutOfRange );
+        PODOFO_RAISE_ERROR( ePdfError_ValueOutOfRange );
     }
 
     const pdf_utf16be* cpUnicodeTable   = this->GetToUnicodeTable();
@@ -103,7 +105,7 @@ pdf_utf16be PdfSimpleEncoding::GetCharCode( int nIndex ) const
 PdfString PdfSimpleEncoding::ConvertToUnicode( const PdfString & rEncodedString, const PdfFont* ) const
 {
     const pdf_utf16be* cpUnicodeTable = this->GetToUnicodeTable();
-    long               lLen           = rEncodedString.GetLength();
+    pdf_long           lLen           = rEncodedString.GetLength();
 
     if( !lLen )
         return PdfString("");
@@ -135,16 +137,16 @@ PdfString PdfSimpleEncoding::ConvertToUnicode( const PdfString & rEncodedString,
     return sStr;
 }
 
-PdfString PdfSimpleEncoding::ConvertToEncoding( const PdfString & rString, const PdfFont* ) const
+PdfRefCountedBuffer PdfSimpleEncoding::ConvertToEncoding( const PdfString & rString, const PdfFont* ) const
 {
     if( !m_pEncodingTable )
         const_cast<PdfSimpleEncoding*>(this)->InitEncodingTable();
 
     PdfString sSrc = rString.ToUnicode(); // make sure the string is unicode and not PdfDocEncoding!
-    long      lLen = sSrc.GetCharacterLength();
+    pdf_long  lLen = sSrc.GetCharacterLength();
 
     if( !lLen )
-        return PdfString("");
+        return PdfRefCountedBuffer();
 
     char* pDest = static_cast<char*>(malloc( sizeof(char) * (lLen + 1) ));
     if( !pDest ) 
@@ -154,6 +156,7 @@ PdfString PdfSimpleEncoding::ConvertToEncoding( const PdfString & rString, const
         
     const pdf_utf16be* pszUtf16 = sSrc.GetUnicode();
     char*              pCur     = pDest;
+    long               lNewLen  = 0L;
 
     for( int i=0;i<lLen;i++ ) 
     {
@@ -163,17 +166,21 @@ PdfString PdfSimpleEncoding::ConvertToEncoding( const PdfString & rString, const
 #endif // PODOFO_IS_LITTLE_ENDIAN
 
         *pCur = m_pEncodingTable[val]; 
-
         if( *pCur ) // ignore 0 characters, as they cannot be converted to the current encoding
+        {
             ++pCur; 
+            ++lNewLen;
+        }
     }
     
     *pCur = '\0';
     
-    PdfString sDest( pDest ); // fake a PdfDocEncoding string .... would be more clear if we return a buffer
+
+    PdfRefCountedBuffer cDest( lNewLen );
+    memcpy( cDest.GetBuffer(), pDest, lNewLen );
     free( pDest );
 
-    return sDest;
+    return cDest;
 }
 
 // -----------------------------------------------------
@@ -859,6 +866,7 @@ const pdf_utf16be PdfMacRomanEncoding::s_cEncoding[256] = {
     0x007C, // VERTICAL LINE
     0x007D, // RIGHT CURLY BRACKET
     0x007E, // TILDE
+    0x007F, // DEL
     0x00C4, // LATIN CAPITAL LETTER A WITH DIAERESIS
     0x00C5, // LATIN CAPITAL LETTER A WITH RING ABOVE
     0x00C7, // LATIN CAPITAL LETTER C WITH CEDILLA
@@ -1055,7 +1063,7 @@ PdfString PdfIdentityEncoding::ConvertToUnicode( const PdfString & rEncodedStrin
     return PdfString( out.str().c_str(), out.str().length() );;
 }
 
-PdfString PdfIdentityEncoding::ConvertToEncoding( const PdfString & rString, const PdfFont* pFont ) const
+PdfRefCountedBuffer PdfIdentityEncoding::ConvertToEncoding( const PdfString & rString, const PdfFont* pFont ) const
 {
     if( !pFont ) 
     {
@@ -1084,7 +1092,9 @@ PdfString PdfIdentityEncoding::ConvertToEncoding( const PdfString & rString, con
         ++pStr;
     }
 
-    return PdfString( out.str().c_str(), out.str().length() );;
+    PdfRefCountedBuffer buffer( out.str().length() );
+    memcpy( buffer.GetBuffer(), out.str().c_str(), out.str().length() );
+    return buffer;
 }
 
 pdf_utf16be PdfIdentityEncoding::GetUnicodeValue( long lCharCode ) const
