@@ -28,6 +28,7 @@
 #include "PdfDictionary.h"
 #include "PdfFilter.h"
 #include "PdfRijndael.h"
+#include "PdfDefinesPrivate.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -35,6 +36,11 @@
 
 namespace PoDoFo {
 
+int PdfEncrypt::s_nEnabledEncryptionAlgorithms = 
+    ePdfEncryptAlgorithm_RC4V1 |
+    ePdfEncryptAlgorithm_RC4V2 |
+    ePdfEncryptAlgorithm_AESV2;
+
 
 /** A class that can encrypt/decrpyt streamed data block wise
  *  This is used in the input and output stream encryption implementation.
@@ -81,10 +87,10 @@ public:
      *  \param pBuffer the input/output buffer. Data is read from this buffer and also stored here
      *  \param lLen    the size of the buffer 
      */
-    long Encrypt( char* pBuffer, long lLen )
+    pdf_long Encrypt( char* pBuffer, pdf_long lLen )
     {
         unsigned char k;
-        int t, i;
+        pdf_long t, i;
 
         // Do not encode data with no length
         if( !lLen )
@@ -97,7 +103,7 @@ public:
             m_b = (m_b + t) % 256;
 
             m_rc4[m_a] = m_rc4[m_b];
-            m_rc4[m_b] = t;
+            m_rc4[m_b] = static_cast<unsigned char>(t);
 
             k = m_rc4[(m_rc4[m_a] + m_rc4[m_b]) % 256];
             pBuffer[i] = pBuffer[i] ^ k;
@@ -133,7 +139,7 @@ public:
      *  \param pBuffer the data is read from this buffer
      *  \param lLen    the size of the buffer 
      */
-    virtual long Write( const char* pBuffer, long lLen )
+    virtual pdf_long Write( const char* pBuffer, pdf_long lLen )
     {
         // Do not encode data with no length
         if( !lLen )
@@ -194,7 +200,7 @@ public:
      *  \returns the number of bytes read, -1 if an error ocurred
      *           and zero if no more bytes are available for reading.
      */
-    virtual long Read( char* pBuffer, long lLen )
+    virtual pdf_long Read( char* pBuffer, pdf_long lLen )
     {
         // Do not encode data with no length
         if( !lLen )
@@ -236,33 +242,6 @@ static void  MD5Init(MD5_CTX *context);
 static void  MD5Update(MD5_CTX *context, unsigned char const *buf, unsigned len);
 static void  MD5Final(unsigned char digest[MD5_HASHBYTES], MD5_CTX *context);
 static void  MD5Transform(unsigned int buf[4], unsigned int const in[16]);
-static char* MD5End(MD5_CTX *, char *);
-
-static char* MD5End(MD5_CTX *ctx, char *buf)
-{
-  int i;
-  unsigned char digest[MD5_HASHBYTES];
-  char hex[]="0123456789abcdef";
-
-  if (!buf)
-  {
-    buf = static_cast<char*>(malloc(33));
-  }
-    
-  if (!buf)
-  {
-    return 0;
-  }
-    
-  MD5Final(digest,ctx);
-  for (i=0;i<MD5_HASHBYTES;i++)
-  {
-    buf[i+i] = hex[digest[i] >> 4];
-    buf[i+i+1] = hex[digest[i] & 0x0f];
-  }
-  buf[i+i] = '\0';
-  return buf;
-}
 
 /*
  * Final wrapup - pad to 64-byte boundary with the bit pattern
@@ -467,29 +446,6 @@ static void MD5Transform(unsigned int buf[4], unsigned int const in[16])
 }
  
 
-#ifndef REVERSEBYTE
-#define byteReverse(buf, len)   /* Nothing */
-#else
-void byteReverse(unsigned char *buf, unsigned longs);
-
-/*
- * Note: this code is harmless on little-endian machines.
- */
-static void byteReverse(unsigned char *buf, unsigned longs)
-{
-  unsigned int t;
-  do
-  {
-    t = (unsigned int) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
-        ((unsigned) buf[1] << 8 | buf[0]);
-    *(unsigned int *) buf = t;  
-    buf += 4;
-  }
-  while (--longs);
-}
-#endif
-
-
 /***************************************************************************
 *   Copyright (C) 2006 by Dominik Seichter                                *
 *   domseichter@web.de                                                    *
@@ -541,20 +497,27 @@ PdfEncrypt::CreatePdfEncrypt( const std::string & userPassword,
     return pdfEncrypt;
 }
 
-PdfEncrypt *
-PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
+PdfEncrypt* PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
 {
-    PdfEncrypt *pdfEncrypt = 0;
+    PdfEncrypt* pdfEncrypt = NULL;
     if( !pObject->GetDictionary().HasKey( PdfName("Filter") ) ||
         pObject->GetDictionary().GetKey( PdfName("Filter" ) )->GetName() != PdfName("Standard") )
     {
         std::ostringstream oss;
-        oss << "Unsupported encryption filter: " << pObject->GetDictionary().GetKey( PdfName("Filter" ) )->GetName().GetName();
+        if( pObject->GetDictionary().HasKey( PdfName("Filter") ) )
+        {
+            oss << "Unsupported encryption filter: " << pObject->GetDictionary().GetKey( PdfName("Filter" ) )->GetName().GetName();
+        }
+        else
+        {
+            oss << "Encryption dictionary does not have a key /Filter.";
+        }
+
         PODOFO_RAISE_ERROR_INFO( ePdfError_UnsupportedFilter, oss.str().c_str() );
     }
         
     long lV;
-    long lLength;
+    long long lLength;
     int rValue;
     int pValue;
     PdfString oValue;
@@ -563,10 +526,10 @@ PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
     try {
         PdfString sTmp;
         
-        lV           = pObject->GetDictionary().GetKey( PdfName("V") )->GetNumber();
-        rValue       = pObject->GetDictionary().GetKey( PdfName("R") )->GetNumber();
+        lV           = static_cast<long>(pObject->GetDictionary().GetKey( PdfName("V") )->GetNumber());
+        rValue       = static_cast<int>(pObject->GetDictionary().GetKey( PdfName("R") )->GetNumber());
 
-        pValue       = pObject->GetDictionary().GetKey( PdfName("P") )->GetNumber();
+        pValue       = static_cast<int>(pObject->GetDictionary().GetKey( PdfName("P") )->GetNumber());
         oValue       = pObject->GetDictionary().GetKey( PdfName("O") )->GetString();
              
         uValue       = pObject->GetDictionary().GetKey( PdfName("U") )->GetString();
@@ -579,15 +542,19 @@ PdfEncrypt::CreatePdfEncrypt( const PdfObject* pObject )
         throw e;
     }
 
-    if( lV == 1L && rValue == 2L ) 
+    if( lV == 1L && rValue == 2L
+        && PdfEncrypt::IsEncryptionEnabled( ePdfEncryptAlgorithm_RC4V1 ) ) 
     {
         pdfEncrypt = new PdfEncryptRC4(oValue, uValue, pValue, rValue, ePdfEncryptAlgorithm_RC4V1, 40);
     }
-    else if( lV == 2L && rValue == 3L ) 
+    else if( lV == 2L && rValue == 3L
+             && PdfEncrypt::IsEncryptionEnabled( ePdfEncryptAlgorithm_RC4V2 ) ) 
     {
-        pdfEncrypt = new PdfEncryptRC4(oValue, uValue, pValue, rValue, ePdfEncryptAlgorithm_RC4V2, lLength);
+			// [Alexey] - lLength is long long. Please make changes in encryption algorithms
+        pdfEncrypt = new PdfEncryptRC4(oValue, uValue, pValue, rValue, ePdfEncryptAlgorithm_RC4V2, static_cast<int>(lLength));
     }
-    else if( lV == 4L && rValue == 4L ) 
+    else if( lV == 4L && rValue == 4L 
+         && PdfEncrypt::IsEncryptionEnabled( ePdfEncryptAlgorithm_AESV2 ) ) 
     {
         pdfEncrypt = new PdfEncryptAES(oValue, uValue, pValue);      
     }
@@ -624,6 +591,21 @@ PdfEncrypt::~PdfEncrypt()
 {
 }
 
+int PdfEncrypt::GetEnabledEncryptionAlgorithms()
+{
+    return PdfEncrypt::s_nEnabledEncryptionAlgorithms;
+}
+
+void PdfEncrypt::SetEnabledEncryptionAlgorithms(int nEncryptionAlgorithms)
+{
+    PdfEncrypt::s_nEnabledEncryptionAlgorithms = nEncryptionAlgorithms;
+}
+
+bool PdfEncrypt::IsEncryptionEnabled(EPdfEncryptAlgorithm eAlgorithm)
+{
+    return (PdfEncrypt::s_nEnabledEncryptionAlgorithms & eAlgorithm);
+}
+
 const PdfEncrypt & PdfEncrypt::operator=( const PdfEncrypt & rhs )
 {
     m_eAlgorithm = rhs.m_eAlgorithm;
@@ -653,12 +635,12 @@ const PdfEncrypt & PdfEncrypt::operator=( const PdfEncrypt & rhs )
 void
 PdfEncrypt::PadPassword(const std::string& password, unsigned char pswd[32])
 {
-  int m = password.length();
+  size_t m = password.length();
 
   if (m > 32) m = 32;
 
-  int j;
-  int p = 0;
+  size_t j;
+  size_t p = 0;
   for (j = 0; j < m; j++)
   {
       pswd[p++] = static_cast<unsigned char>( password[j] );
@@ -823,7 +805,7 @@ PdfEncrypt::ComputeEncryptionKey(const std::string& documentId,
   ext[3] = static_cast<unsigned char> ((pValue >> 24) & 0xff);
   MD5Update(&ctx, ext, 4);
 
-  size_t docIdLength = documentId.length();
+  unsigned int docIdLength = static_cast<unsigned int>(documentId.length());
   unsigned char* docId = NULL;
   if (docIdLength > 0)
   {
@@ -904,7 +886,7 @@ PdfEncrypt::CheckKey(unsigned char key1[32], unsigned char key2[32])
 }
 
 void
-PdfEncrypt::Encrypt(std::string& str, int inputLen) const
+PdfEncrypt::Encrypt(std::string& str, pdf_long inputLen) const
 {
   size_t len = str.length();
   unsigned char* data = new unsigned char[len];
@@ -959,7 +941,7 @@ void PdfEncrypt::CreateObjKey( unsigned char objkey[16], int* pnKeyLen ) const
 
 void
 PdfEncrypt::RC4(unsigned char* key, int keylen,
-                unsigned char* textin, int textlen,
+                unsigned char* textin, pdf_long textlen,
                 unsigned char* textout)
 {
   int i;
@@ -1015,16 +997,17 @@ PdfEncrypt::GetMD5Binary(const unsigned char* data, int length, unsigned char* d
 
 void PdfEncrypt::GenerateInitialVector(unsigned char iv[16])
 {
-    GetMD5Binary(reinterpret_cast<const unsigned char*>(m_documentId.c_str()), m_documentId.length(), iv);
+    GetMD5Binary(reinterpret_cast<const unsigned char*>(m_documentId.c_str()), 
+                 static_cast<unsigned int>(m_documentId.length()), iv);
 }
 
-int
-PdfEncrypt::CalculateStreamLength(int length) const
+pdf_long
+PdfEncrypt::CalculateStreamLength(pdf_long length) const
 {
 	return length;
 }
 
-int PdfEncrypt::CalculateStreamOffset() const
+pdf_long PdfEncrypt::CalculateStreamOffset() const
 {
 	return 0;
 }
@@ -1039,7 +1022,7 @@ PdfString PdfEncrypt::GetMD5String( const unsigned char* pBuffer, int nLength )
 }
 
 void
-PdfEncryptAES::Encrypt(unsigned char* str, int len) const
+PdfEncryptAES::Encrypt(unsigned char* str, pdf_long len) const
 {
   unsigned char objkey[MD5_HASHBYTES];
   int keylen;
@@ -1091,10 +1074,10 @@ PdfEncryptAES::~PdfEncryptAES()
 	delete m_aes;   
 }
 
-int
-PdfEncryptAES::CalculateStreamLength(int length) const
+pdf_long
+PdfEncryptAES::CalculateStreamLength(pdf_long length) const
 {
-  int realLength = length;
+  pdf_long realLength = length;
 //  realLength = (length % 0x7ffffff0) + 32;
   realLength = ((length + 15) & ~15) + 16;
   if (length % 16 == 0)
@@ -1107,13 +1090,13 @@ PdfEncryptAES::CalculateStreamLength(int length) const
 
 void
 PdfEncryptAES::AES(unsigned char* key, int,
-                unsigned char* textin, int textlen,
+                unsigned char* textin, pdf_long textlen,
                 unsigned char* textout)
 {
   GenerateInitialVector(textout);
   m_aes->init( PdfRijndael::CBC, PdfRijndael::Encrypt, key, PdfRijndael::Key16Bytes, textout);
-  int offset = CalculateStreamOffset();
-  int len = m_aes->padEncrypt(&textin[offset], textlen, &textout[offset]);
+  pdf_long offset = CalculateStreamOffset();
+  pdf_long len = m_aes->padEncrypt(&textin[offset], textlen, &textout[offset]);
   
   // It is a good idea to check the error code
   if (len < 0)
@@ -1122,7 +1105,7 @@ PdfEncryptAES::AES(unsigned char* key, int,
   }
 }
 
-int
+pdf_long
 PdfEncryptAES::CalculateStreamOffset() const
 { 
   return 16;
@@ -1144,14 +1127,14 @@ void PdfEncryptAES::CreateEncryptionDictionary( PdfDictionary & rDictionary ) co
 {
     rDictionary.AddKey( PdfName("Filter"), PdfName("Standard") );
    
-    rDictionary.AddKey( PdfName("V"), 4L );
-    rDictionary.AddKey( PdfName("R"), 4L );
-    rDictionary.AddKey( PdfName("Length"), 128L );
+    rDictionary.AddKey( PdfName("V"), static_cast<pdf_int64>(4LL) );
+    rDictionary.AddKey( PdfName("R"), static_cast<pdf_int64>(4LL) );
+    rDictionary.AddKey( PdfName("Length"), static_cast<pdf_int64>(128LL) );
 
     PdfDictionary cf;
     PdfDictionary stdCf;
     stdCf.AddKey( PdfName("CFM"), PdfName("AESV2") );
-    stdCf.AddKey( PdfName("Length"), 16L );
+    stdCf.AddKey( PdfName("Length"), static_cast<pdf_int64>(16LL) );
     stdCf.AddKey( PdfName("AuthEvent"), PdfName("DocOpen") );
     cf.AddKey( PdfName("StdCF"), stdCf );
 
@@ -1161,7 +1144,7 @@ void PdfEncryptAES::CreateEncryptionDictionary( PdfDictionary & rDictionary ) co
 
     rDictionary.AddKey( PdfName("O"), PdfString( reinterpret_cast<const char*>(this->GetOValue()), 32, true ) );
     rDictionary.AddKey( PdfName("U"), PdfString( reinterpret_cast<const char*>(this->GetUValue()), 32, true ) );
-    rDictionary.AddKey( PdfName("P"), PdfVariant( static_cast<long>(this->GetPValue()) ) );
+    rDictionary.AddKey( PdfName("P"), PdfVariant( static_cast<pdf_int64>(this->GetPValue()) ) );
 }
 
 
@@ -1178,7 +1161,7 @@ PdfOutputStream* PdfEncryptAES::CreateEncryptionOutputStream( PdfOutputStream* )
 }
 
 void
-PdfEncryptRC4::Encrypt(unsigned char* str, int len) const
+PdfEncryptRC4::Encrypt(unsigned char* str, pdf_long len) const
 {
   unsigned char objkey[MD5_HASHBYTES];
   int keylen;
@@ -1263,13 +1246,13 @@ void PdfEncryptRC4::CreateEncryptionDictionary( PdfDictionary & rDictionary ) co
     switch( m_eAlgorithm ) 
     {        
         case ePdfEncryptAlgorithm_RC4V1:
-            rDictionary.AddKey( PdfName("V"), 1L );
-            rDictionary.AddKey( PdfName("R"), 2L );
+            rDictionary.AddKey( PdfName("V"), static_cast<pdf_int64>(1LL) );
+            rDictionary.AddKey( PdfName("R"), static_cast<pdf_int64>(2LL) );
             break;
         case ePdfEncryptAlgorithm_RC4V2:
-            rDictionary.AddKey( PdfName("V"), 2L );
-            rDictionary.AddKey( PdfName("R"), 3L );
-            rDictionary.AddKey( PdfName("Length"), PdfVariant( static_cast<long>(m_eKeyLength) ) );
+            rDictionary.AddKey( PdfName("V"), static_cast<pdf_int64>(2LL) );
+            rDictionary.AddKey( PdfName("R"), static_cast<pdf_int64>(3LL) );
+            rDictionary.AddKey( PdfName("Length"), PdfVariant( static_cast<pdf_int64>(m_eKeyLength) ) );
             break;        
 	default:
 	case ePdfEncryptAlgorithm_AESV2:
@@ -1278,7 +1261,7 @@ void PdfEncryptRC4::CreateEncryptionDictionary( PdfDictionary & rDictionary ) co
 
     rDictionary.AddKey( PdfName("O"), PdfString( reinterpret_cast<const char*>(this->GetOValue()), 32, true ) );
     rDictionary.AddKey( PdfName("U"), PdfString( reinterpret_cast<const char*>(this->GetUValue()), 32, true ) );
-    rDictionary.AddKey( PdfName("P"), PdfVariant( static_cast<long>(this->GetPValue()) ) );
+    rDictionary.AddKey( PdfName("P"), PdfVariant( static_cast<pdf_int64>(this->GetPValue()) ) );
 }
 
 
