@@ -1,5 +1,5 @@
-/***************************************************************************
- *   Copyright (C) 2006 by Dominik Seichter                                *
+/**************************************************************************
+ *    Copyright (C) 2006 by Dominik Seichter                                *
  *   domseichter@web.de                                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,7 +18,10 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#include "PdfError.h"
+// PdfError.h doesn't, and can't, include PdfDefines.h so we do so here.
+// PdfDefines.h will include PdfError.h for us.
+#include "PdfDefines.h"
+#include "PdfDefinesPrivate.h"
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -128,31 +131,31 @@ void PdfError::PrintErrorMsg() const
 
     int i                = 0;
 
-    PdfError::LogMessage( eLogSeverity_Error, "\n\nPoDoFo encounter an error. Error: %i %s\n", m_error, pszName ? pszName : "" );
+    PdfError::LogErrorMessage( eLogSeverity_Error, "\n\nPoDoFo encounter an error. Error: %i %s\n", m_error, pszName ? pszName : "" );
 
     if( pszMsg )
-        PdfError::LogMessage( eLogSeverity_Error, "\tError Description: %s\n", pszMsg );
+        PdfError::LogErrorMessage( eLogSeverity_Error, "\tError Description: %s\n", pszMsg );
     
     if( m_callStack.size() )
-        PdfError::LogMessage( eLogSeverity_Error, "\tCallstack:\n" );
+        PdfError::LogErrorMessage( eLogSeverity_Error, "\tCallstack:\n" );
 
     while( it != m_callStack.end() )
     {
         if( !(*it).GetFilename().empty() )
-            PdfError::LogMessage( eLogSeverity_Error, "\t#%i Error Source: %s:%i\n", i, (*it).GetFilename().c_str(), (*it).GetLine() );
+            PdfError::LogErrorMessage( eLogSeverity_Error, "\t#%i Error Source: %s:%i\n", i, (*it).GetFilename().c_str(), (*it).GetLine() );
 
         if( !(*it).GetInformation().empty() )
-            PdfError::LogMessage( eLogSeverity_Error, "\t\tInformation: %s\n", (*it).GetInformation().c_str() );
+            PdfError::LogErrorMessage( eLogSeverity_Error, "\t\tInformation: %s\n", (*it).GetInformation().c_str() );
 
         if( !(*it).GetInformationW().empty() )
-            PdfError::LogMessage( eLogSeverity_Error, L"\t\tInformation: %s\n", (*it).GetInformationW().c_str() );
+            PdfError::LogErrorMessage( eLogSeverity_Error, L"\t\tInformation: %s\n", (*it).GetInformationW().c_str() );
 
         ++i;
         ++it;
     }
 
         
-    PdfError::LogMessage( eLogSeverity_Error, "\n\n" );
+    PdfError::LogErrorMessage( eLogSeverity_Error, "\n\n" );
 }
 
 const char* PdfError::what() const throw()
@@ -195,6 +198,10 @@ const char* PdfError::ErrorName( EPdfError eCode )
             break;
         case ePdfError_InvalidEnumValue:
             pszMsg = "ePdfError_InvalidEnumValue";
+            break;
+        case ePdfError_PageNotFound:
+            pszMsg = "ePdfError_PageNotFound";
+            break;
         case ePdfError_NoPdfFile:
             pszMsg = "ePdfError_NoPdfFile"; 
             break;
@@ -270,6 +277,9 @@ const char* PdfError::ErrorName( EPdfError eCode )
         case ePdfError_ActionAlreadyPresent:
             pszMsg = "ePdfError_ActionAlreadyPresent"; 
             break;
+        case ePdfError_WrongDestinationType:
+            pszMsg = "ePdfError_WrongDestinationType";
+            break;
         case ePdfError_MissingEndStream:
             pszMsg = "ePdfError_MissingEndStream"; 
             break;
@@ -291,9 +301,18 @@ const char* PdfError::ErrorName( EPdfError eCode )
         case ePdfError_UnsupportedImageFormat:    /**< This image format is not supported by PoDoFO. */
             pszMsg = "ePdfError_UnsupportedImageFormat";
             break;
+        case ePdfError_CannotConvertColor:       /**< This color format cannot be converted. */
+            pszMsg = "ePdfError_CannotConvertColor";
+            break;
         case ePdfError_NotImplemented:
             pszMsg = "ePdfError_NotImplemented";
             break;
+        case ePdfError_DestinationAlreadyPresent:
+            pszMsg = "ePdfError_DestinationAlreadyPresent"; 
+            break;
+        case ePdfError_ChangeOnImmutable:
+            pszMsg = "ePdfError_ChangeOnImmutable";
+            break;
         case ePdfError_Unknown:
             pszMsg = "ePdfError_Unknown"; 
             break;
@@ -340,6 +359,9 @@ const char* PdfError::ErrorMessage( EPdfError eCode )
         case ePdfError_InvalidEnumValue:
             pszMsg = "An invalid enum value was specified.";
             break;
+        case ePdfError_PageNotFound:
+            pszMsg = "The requested page could not be found in the PDF.";
+            break;
         case ePdfError_NoPdfFile:
             pszMsg = "This is not a PDF file.";
             break;
@@ -387,8 +409,13 @@ const char* PdfError::ErrorMessage( EPdfError eCode )
         case ePdfError_UnsupportedFontFormat:
             pszMsg = "This font format is not supported by PoDoFO.";
             break;
+        case ePdfError_DestinationAlreadyPresent:
         case ePdfError_ActionAlreadyPresent:
-
+            pszMsg = "Outlines can have either destinations or actions."; 
+            break;
+        case ePdfError_WrongDestinationType:
+            pszMsg = "The requested field is not available for the given destination type";
+            break;
         case ePdfError_MissingEndStream:
         case ePdfError_Date:
             break;
@@ -407,11 +434,18 @@ const char* PdfError::ErrorMessage( EPdfError eCode )
         case ePdfError_UnsupportedImageFormat:
             pszMsg = "This image format is not supported by PoDoFO.";
             break;
+        case ePdfError_CannotConvertColor:
+            pszMsg = "This color format cannot be converted.";
+            break;
+        case ePdfError_ChangeOnImmutable:
+            pszMsg = "Changing values on immutable objects is not allowed.";
+            break;
         case ePdfError_NotImplemented:
             pszMsg = "This feature is currently not implemented.";
             break;
         case ePdfError_Unknown:
             pszMsg = "Error code unknown.";
+            break;
         default:
             break;
     }
@@ -433,6 +467,24 @@ void PdfError::LogMessage( ELogSeverity eLogSeverity, const char* pszMsg, ... )
     if( eLogSeverity < eMinSeverity )
         return;
 
+    va_list  args;
+    va_start( args, pszMsg );
+
+    LogMessageInternal( eLogSeverity, pszMsg, args );
+    va_end( args );
+}
+
+void PdfError::LogErrorMessage( ELogSeverity eLogSeverity, const char* pszMsg, ... )
+{
+    va_list  args;
+    va_start( args, pszMsg );
+
+    LogMessageInternal( eLogSeverity, pszMsg, args );
+    va_end( args );
+}
+
+void PdfError::LogMessageInternal( ELogSeverity eLogSeverity, const char* pszMsg, va_list & args )
+{
     const char* pszPrefix = NULL;
 
     switch( eLogSeverity ) 
@@ -456,20 +508,44 @@ void PdfError::LogMessage( ELogSeverity eLogSeverity, const char* pszMsg, ... )
             break;
     }
 
-    va_list  args;
-    va_start( args, pszMsg );
-    
     if( pszPrefix )
-        fprintf( stderr, pszPrefix );
+        fprintf( stderr, "%s", pszPrefix );
 
     vfprintf( stderr, pszMsg, args );
-    va_end( args );
 }
 
 void PdfError::LogMessage( ELogSeverity eLogSeverity, const wchar_t* pszMsg, ... )
 {
 	if(!PdfError::LoggingEnabled())
 		return;
+
+#ifdef DEBUG
+    const ELogSeverity eMinSeverity = eLogSeverity_Debug;
+#else
+    const ELogSeverity eMinSeverity = eLogSeverity_Information;
+#endif // DEBUG
+
+    if( eLogSeverity < eMinSeverity )
+        return;
+
+    va_list  args;
+    va_start( args, pszMsg );
+
+    LogMessageInternal( eLogSeverity, pszMsg, args );
+    va_end( args );
+}
+
+void PdfError::LogErrorMessage( ELogSeverity eLogSeverity, const wchar_t* pszMsg, ... )
+{
+    va_list  args;
+    va_start( args, pszMsg );
+
+    LogMessageInternal( eLogSeverity, pszMsg, args );
+    va_end( args );
+}
+
+void PdfError::LogMessageInternal( ELogSeverity eLogSeverity, const wchar_t* pszMsg, va_list & args )
+{
     const wchar_t* pszPrefix = NULL;
 
     switch( eLogSeverity ) 
@@ -493,14 +569,10 @@ void PdfError::LogMessage( ELogSeverity eLogSeverity, const wchar_t* pszMsg, ...
             break;
     }
 
-    va_list  args;
-    va_start( args, pszMsg );
-    
     if( pszPrefix )
         fwprintf( stderr, pszPrefix );
 
     vfwprintf( stderr, pszMsg, args );
-    va_end( args );
 }
 
 void PdfError::DebugMessage( const char* pszMsg, ... )
@@ -514,7 +586,7 @@ void PdfError::DebugMessage( const char* pszMsg, ... )
 	va_start( args, pszMsg );
 
 	if( pszPrefix )
-		fprintf( stderr, pszPrefix );
+		fprintf( stderr, "%s", pszPrefix );
 
 	vfprintf( stderr, pszMsg, args );
 	va_end( args );
