@@ -27,6 +27,7 @@
 #include "podofoapi.h"
 #include <string>
 #include <queue>
+#include <cstdarg>
 
 #if defined(_MSC_VER)  &&  _MSC_VER <= 1200 // same pragma as in PdfDefines.h which we cannot include here
 #pragma warning(disable: 4251)
@@ -59,6 +60,8 @@ enum EPdfError {
     ePdfError_InternalLogic,            /**< An internal sanity check or assertion failed. */ 
     ePdfError_InvalidEnumValue,         /**< An invalid enum value was specified. */
 
+    ePdfError_PageNotFound,             /**< The requested page could not be found in the PDF. */
+
     ePdfError_NoPdfFile,                /**< The file is no PDF file. */
     ePdfError_NoXRef,                   /**< The PDF file has no or an invalid XRef table. */
     ePdfError_NoTrailer,                /**< The PDF file has no or an invalid trailer. */
@@ -86,6 +89,7 @@ enum EPdfError {
     ePdfError_UnsupportedFilter,        /**< The requested filter is not yet implemented. */
     ePdfError_UnsupportedFontFormat,    /**< This font format is not supported by PoDoFO. */
     ePdfError_ActionAlreadyPresent,     /**< An Action was already present when trying to add a Destination */
+    ePdfError_WrongDestinationType,     /**< The requested field is not available for the given destination type */
 
     ePdfError_MissingEndStream,         /**< The required token endstream was not found. */
     ePdfError_Date,                     /**< Date/time error */
@@ -96,9 +100,13 @@ enum EPdfError {
     ePdfError_MutexError,               /**< Error during a mutex operation */
 
     ePdfError_UnsupportedImageFormat,   /**< This image format is not supported by PoDoFO. */
+    ePdfError_CannotConvertColor,       /**< This color format cannot be converted. */
 
     ePdfError_NotImplemented,           /**< This feature is currently not implemented. */
 
+    ePdfError_DestinationAlreadyPresent,/**< An destination was already present when trying to add a Action */
+    ePdfError_ChangeOnImmutable,        /**< Changing values on immutable objects is not allowed. */
+
     ePdfError_Unknown = 0xffff          /**< Unknown error */
 };
 
@@ -372,6 +380,30 @@ class PODOFO_EXCEPTION_API_DOXYGEN PdfError : public std::exception {
      */
     static bool DebugEnabled() { return PdfError::s_DgbEnabled; }
 
+ private:
+    /** Log a message to the logging system defined for PoDoFo.
+     *
+     *  This call does not check if logging is enabled and always
+     *  prints the error message
+     *
+     *  \param eLogSeverity the sevirity of the log message
+     *  \param pszMsg       the message to be logged
+     */
+    static void LogErrorMessage( ELogSeverity eLogSeverity, const char* pszMsg, ... );
+
+    /** Log a message to the logging system defined for PoDoFo.
+     *
+     *  This call does not check if logging is enabled and always
+     *  prints the error message
+     *
+     *  \param eLogSeverity the sevirity of the log message
+     *  \param pszMsg       the message to be logged
+     */
+    static void LogErrorMessage( ELogSeverity eLogSeverity, const wchar_t* pszMsg, ... );
+
+    static void LogMessageInternal( ELogSeverity eLogSeverity, const char* pszMsg, va_list & args );
+    static void LogMessageInternal( ELogSeverity eLogSeverity, const wchar_t* pszMsg, va_list & args );
+
  private:
     EPdfError          m_error;
 
