@@ -21,18 +21,21 @@
 #include "PdfFontCache.h" 
 
 #include "PdfDifferenceEncoding.h"
+#include "PdfDictionary.h"
 #include "PdfFont.h"
 #include "PdfFontFactory.h"
-#include "PdfFontMetrics.h"
+#include "PdfFontMetricsFreetype.h"
+#include "PdfFontMetricsBase14.h"
 #include "PdfFontTTFSubset.h"
+#include "PdfFontType1.h"
 #include "PdfInputDevice.h"
 #include "PdfOutputDevice.h"
+#include "PdfDefinesPrivate.h"
 
 #include <algorithm>
 
 #ifdef _WIN32
-#include <windows.h>
-
+//#include <windows.h>
 // Undefined stuff which windows does
 // define that breaks are build
 // e.g. GetObject is defined to either GetObjectA or GetObjectW
@@ -49,8 +52,9 @@
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
-#if defined(HAVE_FONTCONFIG)
+#if defined(PODOFO_HAVE_FONTCONFIG)
 #include <fontconfig/fontconfig.h>
+#include "util/PdfMutexWrapper.h"
 #endif
 
 using namespace std;
@@ -67,11 +71,13 @@ static bool GetDataFromHFONT( HFONT hf, char** outFontBuffer, unsigned int& outF
         return false;
     }
     
-    SelectObject(hdc, hf);
+    // Petr Petrov (22 December 2009)
+    HGDIOBJ oldFont = SelectObject(hdc, hf);
 
     outFontBufferLen = GetFontData(hdc, 0, 0, 0, 0);
     
     if (outFontBufferLen == GDI_ERROR) {
+        SelectObject(hdc,oldFont);
         ReleaseDC(0, hdc);
         DeleteObject(hf);
         return false;
@@ -83,11 +89,13 @@ static bool GetDataFromHFONT( HFONT hf, char** outFontBuffer, unsigned int& outF
         free( *outFontBuffer );
         *outFontBuffer = NULL;
         outFontBufferLen = 0;
+        SelectObject(hdc,oldFont);
         ReleaseDC(0, hdc);
         DeleteObject(hf);
         return false;
     }
     
+    SelectObject(hdc,oldFont);
     ReleaseDC( 0, hdc );
     DeleteObject( hf );
     
@@ -115,13 +123,20 @@ static bool GetDataFromLPFONT( const LOGFONTW* inFont, char** outFontBuffer, uns
 }
 #endif // _WIN32
 
+#if defined(PODOFO_HAVE_FONTCONFIG)
+Util::PdfMutex PdfFontCache::m_FcMutex;
+#endif
+
 PdfFontCache::PdfFontCache( PdfVecObjects* pParent )
     : m_pParent( pParent )
 {
     // Initialize all the fonts stuff
 
-#if defined(HAVE_FONTCONFIG)
-    m_pFcConfig     = static_cast<void*>(FcInitLoadConfigAndFonts());
+#if defined(PODOFO_HAVE_FONTCONFIG)
+    {
+        Util::PdfMutexWrapper mutex(m_FcMutex);
+        m_pFcConfig     = static_cast<void*>(FcInitLoadConfigAndFonts());
+    }
 #endif
 
     if( FT_Init_FreeType( &m_ftLibrary ) )
@@ -134,8 +149,11 @@ PdfFontCache::~PdfFontCache()
 {
     this->EmptyCache();
 
-#if defined(HAVE_FONTCONFIG)
-    FcConfigDestroy( static_cast<FcConfig*>(m_pFcConfig) );
+#if defined(PODOFO_HAVE_FONTCONFIG)
+    {
+        Util::PdfMutexWrapper mutex(m_FcMutex);
+        FcConfigDestroy( static_cast<FcConfig*>(m_pFcConfig) );
+    }
 #endif
 
     if( m_ftLibrary ) 
@@ -215,37 +233,69 @@ PdfFont* PdfFontCache::GetFont( const char* pszFontName, bool bBold, bool bItali
 {
     PODOFO_ASSERT( pEncoding );
 
-    PdfFont*          pFont;
-    PdfFontMetrics*   pMetrics;
+    PdfFont*          pFont = NULL;
+    PdfFontMetrics*   pMetrics = NULL;
     std::pair<TISortedFontList,TCISortedFontList> it;
 
     it = std::equal_range( m_vecFonts.begin(), m_vecFonts.end(), 
 			   TFontCacheElement( pszFontName, bBold, bItalic, pEncoding ) );
+
+		
     if( it.first == it.second )
     {
-        std::string sPath;
-        if ( pszFileName == NULL )
-            sPath = this->GetFontPath( pszFontName, bBold, bItalic );
-        else
-            sPath = pszFileName;
-        
-        if( sPath.empty() )
-        {
+		if (PODOFO_Base14FontDef_FindBuiltinData(pszFontName))
+		{  
+			pFont = CreateBase14Font(pszFontName,pEncoding,m_pParent);
+
+			if( pFont ) 
+			{
+				TFontCacheElement element;
+				element.m_pFont     = pFont;
+				element.m_bBold     = pFont->IsBold();
+				element.m_bItalic   = pFont->IsItalic();
+				element.m_sFontName = pszFontName;
+				element.m_pEncoding = pEncoding;
+
+				// Do a sorted insert, so no need to sort again
+				//rvecContainer.insert( itSorted, element ); 
+				m_vecFonts.insert( it.first, element );
+				
+			 }
+
+	//	pdfont = new PdfFontType1(PODOFO_Base14FontDef_FindBuiltinData(pszFontName), pEncoding, &m_vecObjects);
+		}
+		
+		if (!pFont)
+		{
+			std::string sPath;
+			if ( pszFileName == NULL )
+				sPath = this->GetFontPath( pszFontName, bBold, bItalic );
+			else
+				sPath = pszFileName;
+	        
+			if( sPath.empty() )
+			{
 #ifdef _WIN32
-            return GetWin32Font( it.first, m_vecFonts, pszFontName, bBold, bItalic, bEmbedd, pEncoding );
-#else
-            PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
-            return NULL;
+				pFont = GetWin32Font( it.first, m_vecFonts, pszFontName, bBold, bItalic, bEmbedd, pEncoding );
 #endif // _WIN32
-        }
-        
-        pMetrics = new PdfFontMetrics( &m_ftLibrary, sPath.c_str() );
-        pFont    = this->CreateFontObject( it.first, m_vecFonts, pMetrics, 
-					   bEmbedd, bBold, bItalic, pszFontName, pEncoding );
+			}
+			else
+			{
+				pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, sPath.c_str() );
+				pFont    = this->CreateFontObject( it.first, m_vecFonts, pMetrics, 
+						   bEmbedd, bBold, bItalic, pszFontName, pEncoding );
+			}
+
+		}
     }
     else
         pFont = (*it.first).m_pFont;
-    
+
+#ifndef WIN32
+		if (!pFont)
+	        PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
+#endif             
+
     return pFont;
 }
 
@@ -289,7 +339,7 @@ PdfFont* PdfFontCache::GetFont( FT_Face face, bool bEmbedd, const PdfEncoding *
 			   TFontCacheElement( sName.c_str(), bBold, bItalic, pEncoding ) );
     if( it.first == it.second )
     {
-        pMetrics = new PdfFontMetrics( &m_ftLibrary, face );
+        pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, face );
         pFont    = this->CreateFontObject( it.first, m_vecFonts, pMetrics, 
 					   bEmbedd, bBold, bItalic, sName.c_str(), pEncoding );
     }
@@ -299,6 +349,55 @@ PdfFont* PdfFontCache::GetFont( FT_Face face, bool bEmbedd, const PdfEncoding *
     return pFont;
 }
 
+PdfFont* PdfFontCache::GetDuplicateFontType1( PdfFont * pFont, const char* pszSuffix )
+{
+    TCISortedFontList it = m_vecFonts.begin();
+
+	std::string id = pFont->GetIdentifier().GetName();
+	id += pszSuffix;
+
+    // Search if the object is a cached normal font
+    while( it != m_vecFonts.end() )
+    {
+		if( (*it).m_pFont->GetIdentifier() == id ) 
+            return (*it).m_pFont;
+
+        ++it;
+    }
+
+    // Search if the object is a cached font subset
+    it = m_vecFontSubsets.begin();
+    while( it != m_vecFontSubsets.end() )
+    {
+        if( (*it).m_pFont->GetIdentifier() == id ) 
+            return (*it).m_pFont;
+
+        ++it;
+    }
+
+    // Create a copy of the font
+	PODOFO_ASSERT( pFont->GetFontMetrics()->GetFontType() == ePdfFontType_Type1Pfb );
+	PdfFontMetrics* pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, pFont->GetFontMetrics()->GetFilename() );
+	PdfFont* newFont = new PdfFontType1( static_cast<PdfFontType1 *>(pFont), pMetrics, pszSuffix, m_pParent );
+    if( newFont ) 
+    {
+		std::string name = newFont->GetFontMetrics()->GetFontname();
+		name += pszSuffix;
+        TFontCacheElement element;
+        element.m_pFont     = newFont;
+        element.m_bBold     = newFont->IsBold();
+        element.m_bItalic   = newFont->IsItalic();
+        element.m_sFontName = name;
+        element.m_pEncoding = newFont->GetEncoding();
+        m_vecFonts  .push_back( element );
+        
+        // Now sort the font list
+        std::sort( m_vecFonts.begin(), m_vecFonts.end() );
+    }
+
+    return newFont;
+}
+
 PdfFont* PdfFontCache::GetFontSubset( const char* pszFontName, bool bBold, bool bItalic, 
 				      const PdfEncoding * const pEncoding,
 				      const char* pszFileName )
@@ -313,62 +412,62 @@ PdfFont* PdfFontCache::GetFontSubset( const char* pszFontName, bool bBold, bool
 			   TFontCacheElement( pszFontName, bBold, bItalic, pEncoding ) );
     if( it.first == it.second )
     {
-	std::string sPath; 
-	if( pszFileName == NULL ) 
-	{
-	    sPath = this->GetFontPath( pszFontName, bBold, bItalic );
-	    if( sPath.empty() )
-	    {
+        std::string sPath; 
+        if( pszFileName == NULL ) 
+        {
+            sPath = this->GetFontPath( pszFontName, bBold, bItalic );
+            if( sPath.empty() )
+            {
 #ifdef _WIN32
-		// TODO: GetWin32Font
-		PODOFO_ASSERT( 0 );
+                // TODO: GetWin32Font
+                PODOFO_ASSERT( 0 );
 #else	    
-		PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
-		return NULL;
+                PdfError::LogMessage( eLogSeverity_Critical, "No path was found for the specified fontname: %s\n", pszFontName );
+                return NULL;
 #endif // _WIN32
-	    }
-	}
-	else
-	    sPath = pszFileName;
-	
-	pMetrics = new PdfFontMetrics( &m_ftLibrary, sPath.c_str() );
-	if( !(pMetrics && pMetrics->GetFontType() == ePdfFontType_TrueType ) )
-	{
-	    PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidFontFile, "Subsetting is only supported for TrueType fonts." );
-	}
-	
-	PdfInputDevice          input( sPath.c_str() );
-	PdfRefCountedBuffer     buffer;
-	PdfOutputDevice         output( &buffer );
-	
-	PdfFontTTFSubset        subset( &input, pMetrics, PdfFontTTFSubset::eFontFileType_TTF );
-	PdfEncoding::const_iterator itChar
-	    = pEncoding->begin();
-	while( itChar != pEncoding->end() )
-	{
-	    subset.AddCharacter( *itChar );
-	    ++itChar;
-	}
-	subset.BuildFont( &output );
-
-	// Delete metrics object, as it was only used so that PdfFontTTFSubset could
-	// match unicode character points to glyph indeces
-	delete pMetrics;
-	// TODO: Do not hardcode unique basenames...
-	pMetrics = new PdfFontMetrics( &m_ftLibrary, buffer, "ABCDEF+" );
-	pFont = this->CreateFontObject( it.first, m_vecFontSubsets, pMetrics, 
-					true, bBold, bItalic, pszFontName, pEncoding );
+            }
+        }
+        else
+            sPath = pszFileName;
+        
+        pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, sPath.c_str() );
+        if( !(pMetrics && pMetrics->GetFontType() == ePdfFontType_TrueType ) )
+        {
+            PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidFontFile, "Subsetting is only supported for TrueType fonts." );
+        }
+        
+        PdfInputDevice          input( sPath.c_str() );
+        PdfRefCountedBuffer     buffer;
+        PdfOutputDevice         output( &buffer );
+        
+        PdfFontTTFSubset        subset( &input, pMetrics, PdfFontTTFSubset::eFontFileType_TTF );
+        PdfEncoding::const_iterator itChar
+            = pEncoding->begin();
+        while( itChar != pEncoding->end() )
+        {
+            subset.AddCharacter( *itChar );
+            ++itChar;
+        }
+        subset.BuildFont( &output );
+        
+        // Delete metrics object, as it was only used so that PdfFontTTFSubset could
+        // match unicode character points to glyph indeces
+        delete pMetrics;
+        // TODO: Do not hardcode unique basenames...
+        pMetrics = new PdfFontMetricsFreetype( &m_ftLibrary, buffer, "ABCDEF+" );
+        pFont = this->CreateFontObject( it.first, m_vecFontSubsets, pMetrics, 
+                                        true, bBold, bItalic, pszFontName, pEncoding );
     }
     else
-	pFont = (*it.first).m_pFont;
-
-
+        pFont = (*it.first).m_pFont;
+    
+    
     return pFont;
 }
 
 #ifdef _WIN32
 PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList & vecContainer, 
-				     const char* pszFontName, bool bBold, bool bItalic, 
+                                     const char* pszFontName, bool bBold, bool bItalic, 
                                      bool bEmbedd, const PdfEncoding * const pEncoding )
 {
     LOGFONT	lf;
@@ -393,7 +492,7 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
     memset(&(lf.lfFaceName), 0, LF_FACESIZE);
     strcpy( static_cast<char*>(lf.lfFaceName), pszFontName );
     
-    char*        pBuffer;
+    char*        pBuffer = NULL;
     unsigned int nLen;
     if( !GetDataFromLPFONT( &lf, &pBuffer, nLen ) )
         return NULL;
@@ -405,11 +504,11 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
         pFont    = this->CreateFontObject( itSorted, vecContainer, pMetrics, 
 					   bEmbedd, bBold, bItalic, pszFontName, pEncoding );
     } catch( PdfError & error ) {
-        //free( pBuffer );
+        free( pBuffer );
         throw error;
     }
     
-    //free( pBuffer );
+    free( pBuffer );
     return pFont;
 }
 
@@ -433,27 +532,30 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
     lf.lfQuality		= DEFAULT_QUALITY;
     lf.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
     
-    long lFontNameLen = wcslen(pszFontName);
+    pdf_long lFontNameLen = wcslen(pszFontName);
     if (lFontNameLen >= LF_FACESIZE)
         return NULL;
     
     memset(&(lf.lfFaceName), 0, LF_FACESIZE);
     wcscpy( static_cast<wchar_t*>(lf.lfFaceName), pszFontName );
     
-    char*        pBuffer;
+    char*        pBuffer = NULL;
     unsigned int nLen;
     if( !GetDataFromLPFONT( &lf, &pBuffer, nLen ) )
         return NULL;
     
-    long lMaxLen = lFontNameLen * 5;
+    pdf_long lMaxLen = lFontNameLen * 5;
     char* pmbFontName = static_cast<char*>(malloc(lMaxLen));
     if( !pmbFontName )
     {
+        free( pBuffer );
         PODOFO_RAISE_ERROR( ePdfError_OutOfMemory );
     }
 
     if( wcstombs( pmbFontName, pszFontName, lMaxLen ) <= 0 )
     {
+        free( pBuffer );
+        free( pmbFontName );
         PODOFO_RAISE_ERROR_INFO( ePdfError_InternalLogic, "Conversion to multibyte char failed" );
     }
 
@@ -468,16 +570,16 @@ PdfFont* PdfFontCache::GetWin32Font( TISortedFontList itSorted, TSortedFontList
     } catch( PdfError & error ) {
         free( pmbFontName );
         pmbFontName = NULL;
-        //free( pBuffer );
+        free( pBuffer );
         throw error;
     }
     
-    //free( pBuffer );
+    free( pBuffer );
     return pFont;
 }
 #endif // _WIN32
 
-#if defined(HAVE_FONTCONFIG)
+#if defined(PODOFO_HAVE_FONTCONFIG)
 std::string PdfFontCache::GetFontConfigFontPath( FcConfig* pConfig, const char* pszFontName, bool bBold, bool bItalic )
 {
     FcPattern*  pattern;
@@ -485,6 +587,7 @@ std::string PdfFontCache::GetFontConfigFontPath( FcConfig* pConfig, const char*
     FcResult    result = FcResultMatch;
     FcValue     v;
     std::string sPath;
+    Util::PdfMutexWrapper mutex(m_FcMutex);
 
     // Build a pattern to search using fontname, bold and italic
     pattern = FcPatternBuild (0, FC_FAMILY, FcTypeString, pszFontName, 
@@ -515,11 +618,12 @@ std::string PdfFontCache::GetFontConfigFontPath( FcConfig* pConfig, const char*
     return sPath;
 }
 
-#endif // HAVE_FONTCONFIG
+#endif // PODOFO_HAVE_FONTCONFIG
 
 std::string PdfFontCache::GetFontPath( const char* pszFontName, bool bBold, bool bItalic )
 {
-#if defined(HAVE_FONTCONFIG)
+#if defined(PODOFO_HAVE_FONTCONFIG)
+    Util::PdfMutexWrapper mutex(m_FcMutex);
     FcConfig*   pConfig = FcInitLoadConfigAndFonts();
     std::string sPath   = this->GetFontConfigFontPath( pConfig, pszFontName, bBold, bItalic );
     FcConfigDestroy( pConfig );    
@@ -550,7 +654,7 @@ PdfFont* PdfFontCache::CreateFontObject( TISortedFontList itSorted, TSortedFontL
         pFont    = PdfFontFactory::CreateFontObject( pMetrics, nFlags, pEncoding, m_pParent );
 
         if( pFont ) 
-	{
+        {
             TFontCacheElement element;
             element.m_pFont     = pFont;
             element.m_bBold     = pFont->IsBold();
@@ -558,9 +662,9 @@ PdfFont* PdfFontCache::CreateFontObject( TISortedFontList itSorted, TSortedFontL
             element.m_sFontName = pszFontName;
             element.m_pEncoding = pEncoding;
 
-	    // Do a sorted insert, so no need to sort again
+            // Do a sorted insert, so no need to sort again
             rvecContainer.insert( itSorted, element );
-	}
+        }
     } catch( PdfError & e ) {
         e.AddToCallstack( __FILE__, __LINE__ );
         e.PrintErrorMsg();
