@@ -22,6 +22,7 @@
 
 #include "PdfDocument.h"
 #include "PdfStream.h"
+#include "PdfDefinesPrivate.h"
 
 #include <stdio.h>
 #include <wchar.h>
@@ -29,20 +30,29 @@
 
 #ifdef PODOFO_HAVE_TIFF_LIB
 extern "C" {
-#include "tiffio.h"
-#ifdef _WIN32		// Collision between tiff and jpeg-headers
-#define XMD_H
-#undef FAR
-#endif
+#  include "tiffio.h"
+#  ifdef _WIN32		// Collision between tiff and jpeg-headers
+#    ifndef XMD_H
+#    define XMD_H
+#    endif
+#    undef FAR
+#  endif
 }
 #endif // PODOFO_HAVE_TIFF_LIB
 
 #ifdef PODOFO_HAVE_JPEG_LIB
 extern "C" {
-#include "jpeglib.h"
+#  ifndef XMD_H
+#    define XMD_H
+#  endif
+#  include "jpeglib.h"
 }
 #endif // PODOFO_HAVE_JPEG_LIB
 
+#ifdef PODOFO_HAVE_PNG_LIB
+#include <png.h>
+#endif /// PODOFO_HAVE_PNG_LIB
+
 // <windows.h> defines an annoying GetObject macro that changes uses of GetObject to
 // GetObjectA . Since macros aren't scope and namespace aware that breaks our code.
 // Since we won't be using the Win32 resource manager API here, just undefine it.
@@ -54,16 +64,16 @@ using namespace std;
 
 namespace PoDoFo {
 
-PdfImage::PdfImage( PdfVecObjects* pParent )
-    : PdfXObject( "Image", pParent )
+PdfImage::PdfImage( PdfVecObjects* pParent, const char* pszPrefix )
+    : PdfXObject( "Image", pParent, pszPrefix )
 {
     m_rRect = PdfRect();
 
     this->SetImageColorSpace( ePdfColorSpace_DeviceRGB );
 }
 
-PdfImage::PdfImage( PdfDocument* pParent )
-    : PdfXObject( "Image", pParent )
+PdfImage::PdfImage( PdfDocument* pParent, const char* pszPrefix )
+    : PdfXObject( "Image", pParent, pszPrefix )
 {
     m_rRect = PdfRect();
 
@@ -73,18 +83,40 @@ PdfImage::PdfImage( PdfDocument* pParent )
 PdfImage::PdfImage( PdfObject* pObject )
     : PdfXObject( "Image", pObject )
 {
-    m_rRect.SetHeight( m_pObject->GetDictionary().GetKey( "Height" )->GetNumber() );
-    m_rRect.SetWidth ( m_pObject->GetDictionary().GetKey( "Width" )->GetNumber() );
+    m_rRect.SetHeight( static_cast<double>(this->GetObject()->GetDictionary().GetKey( "Height" )->GetNumber()) );
+    m_rRect.SetWidth ( static_cast<double>(this->GetObject()->GetDictionary().GetKey( "Width" )->GetNumber()) );
 }
 
 PdfImage::~PdfImage()
 {
 
+}
+
+     /* Example: { "JPEG", "TIFF", NULL }
+     *
+     * \returns a zero terminates list of all supported image formats
+     */
+const char** PdfImage::GetSupportedFormats()
+{
+    static const char* ppszFormats[] = {
+#ifdef PODOFO_HAVE_JPEG_LIB
+        "JPEG",
+#endif // PODOFO_HAVE_JPEG_LIB
+#ifdef PODOFO_HAVE_PNG_LIB
+        "PNG", 
+#endif // PODOFO_HAVE_PNG_LIB
+#ifdef PODOFO_HAVE_TIFF_LIB
+        "TIFF", 
+#endif // PODOFO_HAVE_TIFF_LIB
+        NULL
+    };
+
+    return ppszFormats;
 }
 
 void PdfImage::SetImageColorSpace( EPdfColorSpace eColorSpace )
 {
-    m_pObject->GetDictionary().AddKey( PdfName("ColorSpace"), PdfName( ColorspaceToName( eColorSpace ) ) );
+    this->GetObject()->GetDictionary().AddKey( PdfName("ColorSpace"), PdfName( ColorspaceToName( eColorSpace ) ) );
 }
 
 void PdfImage::SetImageICCProfile( PdfInputStream* pStream, long lColorComponents, EPdfColorSpace eAlternateColorSpace ) 
@@ -100,7 +132,7 @@ void PdfImage::SetImageICCProfile( PdfInputStream* pStream, long lColorComponent
     // Create a colorspace object
     PdfObject* pIccObject = this->GetObject()->GetOwner()->CreateObject();
     pIccObject->GetDictionary().AddKey( PdfName("Alternate"), PdfName( ColorspaceToName( eAlternateColorSpace ) ) );
-    pIccObject->GetDictionary().AddKey( PdfName("N"), lColorComponents );
+    pIccObject->GetDictionary().AddKey( PdfName("N"), static_cast<pdf_int64>(lColorComponents) );
     pIccObject->GetStream()->Set( pStream );
     
     // Add the colorspace to our image
@@ -110,6 +142,11 @@ void PdfImage::SetImageICCProfile( PdfInputStream* pStream, long lColorComponent
     this->GetObject()->GetDictionary().AddKey( PdfName("ColorSpace"), array );
 }
 
+void PdfImage::SetImageSoftmask( const PdfImage* pSoftmask )
+{
+	GetObject()->GetDictionary().AddKey( "SMask", pSoftmask->GetObject()->Reference() );
+}
+
 void PdfImage::SetImageData( unsigned int nWidth, unsigned int nHeight, 
                              unsigned int nBitsPerComponent, PdfInputStream* pStream )
 {
@@ -126,15 +163,15 @@ void PdfImage::SetImageData( unsigned int nWidth, unsigned int nHeight,
     m_rRect.SetWidth( nWidth );
     m_rRect.SetHeight( nHeight );
 
-    m_pObject->GetDictionary().AddKey( "Width",  PdfVariant( static_cast<long>(nWidth) ) );
-    m_pObject->GetDictionary().AddKey( "Height", PdfVariant( static_cast<long>(nHeight) ) );
-    m_pObject->GetDictionary().AddKey( "BitsPerComponent", PdfVariant( static_cast<long>(nBitsPerComponent) ) );
+    this->GetObject()->GetDictionary().AddKey( "Width",  PdfVariant( static_cast<pdf_int64>(nWidth) ) );
+    this->GetObject()->GetDictionary().AddKey( "Height", PdfVariant( static_cast<pdf_int64>(nHeight) ) );
+    this->GetObject()->GetDictionary().AddKey( "BitsPerComponent", PdfVariant( static_cast<pdf_int64>(nBitsPerComponent) ) );
 
     PdfVariant var;
     m_rRect.ToVariant( var );
-    m_pObject->GetDictionary().AddKey( "BBox", var );
+    this->GetObject()->GetDictionary().AddKey( "BBox", var );
 
-    m_pObject->GetStream()->Set( pStream, vecFilters );
+    this->GetObject()->GetStream()->Set( pStream, vecFilters );
 }
 
 void PdfImage::SetImageDataRaw( unsigned int nWidth, unsigned int nHeight, 
@@ -143,15 +180,15 @@ void PdfImage::SetImageDataRaw( unsigned int nWidth, unsigned int nHeight,
     m_rRect.SetWidth( nWidth );
     m_rRect.SetHeight( nHeight );
 
-    m_pObject->GetDictionary().AddKey( "Width",  PdfVariant( static_cast<long>(nWidth) ) );
-    m_pObject->GetDictionary().AddKey( "Height", PdfVariant( static_cast<long>(nHeight) ) );
-    m_pObject->GetDictionary().AddKey( "BitsPerComponent", PdfVariant( static_cast<long>(nBitsPerComponent) ) );
+    this->GetObject()->GetDictionary().AddKey( "Width",  PdfVariant( static_cast<pdf_int64>(nWidth) ) );
+    this->GetObject()->GetDictionary().AddKey( "Height", PdfVariant( static_cast<pdf_int64>(nHeight) ) );
+    this->GetObject()->GetDictionary().AddKey( "BitsPerComponent", PdfVariant( static_cast<pdf_int64>(nBitsPerComponent) ) );
 
     PdfVariant var;
     m_rRect.ToVariant( var );
-    m_pObject->GetDictionary().AddKey( "BBox", var );
+    this->GetObject()->GetDictionary().AddKey( "BBox", var );
 
-    m_pObject->GetStream()->SetRawData( pStream, -1 );
+    this->GetObject()->GetStream()->SetRawData( pStream, -1 );
 }
 
 void PdfImage::LoadFromFile( const char* pszFilename )
@@ -161,13 +198,8 @@ void PdfImage::LoadFromFile( const char* pszFilename )
         const char* pszExtension = pszFilename + strlen( pszFilename ) - 3;
 
 #ifdef PODOFO_HAVE_TIFF_LIB
-#ifdef _MSC_VER
-        if( _strnicmp( pszExtension, "tif", 3 ) == 0 || 
-            _strnicmp( pszExtension, "iff", 3 ) == 0 ) // "tiff"
-#else
-        if( strncasecmp( pszExtension, "tif", 3 ) == 0 ||
-            strncasecmp( pszExtension, "iff", 3 ) == 0 ) // "tiff"
-#endif
+        if( PoDoFo::compat::strncasecmp( pszExtension, "tif", 3 ) == 0 ||
+            PoDoFo::compat::strncasecmp( pszExtension, "iff", 3 ) == 0 ) // "tiff"
         {
             LoadFromTiff( pszFilename );
             return;
@@ -175,18 +207,23 @@ void PdfImage::LoadFromFile( const char* pszFilename )
 #endif
 
 #ifdef PODOFO_HAVE_JPEG_LIB
-#ifdef _MSC_VER
-        if( _strnicmp( pszExtension, "jpg", 3 ) == 0 )
-#else
-        if( strncasecmp( pszExtension, "jpg", 3 ) == 0 )
-#endif
+        if( PoDoFo::compat::strncasecmp( pszExtension, "jpg", 3 ) == 0 )
         {
             LoadFromJpeg( pszFilename );
             return;
         }
 #endif
+
+#ifdef PODOFO_HAVE_PNG_LIB
+        if( PoDoFo::compat::strncasecmp( pszExtension, "png", 3 ) == 0 )
+        {
+            LoadFromPng( pszFilename );
+            return;
+        }
+#endif
+
 	}
-	PODOFO_RAISE_ERROR( ePdfError_UnsupportedImageFormat );
+	PODOFO_RAISE_ERROR_INFO( ePdfError_UnsupportedImageFormat, pszFilename );
 }
 
 #ifdef PODOFO_HAVE_JPEG_LIB
@@ -314,7 +351,7 @@ void PdfImage::LoadFromJpegHandle( FILE* hInfile, PdfFileInputStream* pInStream
     }
     
     // Set the filters key to DCTDecode
-    m_pObject->GetDictionary().AddKey( PdfName::KeyFilter, PdfName( "DCTDecode" ) );
+    this->GetObject()->GetDictionary().AddKey( PdfName::KeyFilter, PdfName( "DCTDecode" ) );
     // Do not apply any filters as JPEG data is already DCT encoded.
     this->SetImageDataRaw( cinfo.output_width, cinfo.output_height, 8, pInStream );
     
@@ -398,11 +435,11 @@ void PdfImage::LoadFromTiff( const char* pszFilename )
 			if( bitsPixel == 1 )
 			{
 				PdfArray decode;
-				decode.insert( decode.end(), PdfVariant( static_cast<long>(0) ) );
-				decode.insert( decode.end(), PdfVariant( static_cast<long>(1) ) );
-				m_pObject->GetDictionary().AddKey( PdfName("Decode"), decode );
-				m_pObject->GetDictionary().AddKey( PdfName("ImageMask"), PdfVariant( true ) );
-				m_pObject->GetDictionary().RemoveKey( PdfName("ColorSpace") );
+				decode.insert( decode.end(), PdfVariant( static_cast<pdf_int64>(0) ) );
+				decode.insert( decode.end(), PdfVariant( static_cast<pdf_int64>(1) ) );
+				this->GetObject()->GetDictionary().AddKey( PdfName("Decode"), decode );
+				this->GetObject()->GetDictionary().AddKey( PdfName("ImageMask"), PdfVariant( true ) );
+				this->GetObject()->GetDictionary().RemoveKey( PdfName("ColorSpace") );
 			}
 			else if ( bitsPixel == 8)
 	            SetImageColorSpace(ePdfColorSpace_DeviceGray);
@@ -419,11 +456,11 @@ void PdfImage::LoadFromTiff( const char* pszFilename )
 			if( bitsPixel == 1 )
 			{
 				PdfArray decode;
-				decode.insert( decode.end(), PdfVariant( static_cast<long>(1) ) );
-				decode.insert( decode.end(), PdfVariant( static_cast<long>(0) ) );
-				m_pObject->GetDictionary().AddKey( PdfName("Decode"), decode );
-				m_pObject->GetDictionary().AddKey( PdfName("ImageMask"), PdfVariant( true ) );
-				m_pObject->GetDictionary().RemoveKey( PdfName("ColorSpace") );
+				decode.insert( decode.end(), PdfVariant( static_cast<pdf_int64>(1) ) );
+				decode.insert( decode.end(), PdfVariant( static_cast<pdf_int64>(0) ) );
+				this->GetObject()->GetDictionary().AddKey( PdfName("Decode"), decode );
+				this->GetObject()->GetDictionary().AddKey( PdfName("ImageMask"), PdfVariant( true ) );
+				this->GetObject()->GetDictionary().RemoveKey( PdfName("ColorSpace") );
 			}
 			else if ( bitsPixel == 8)
 	            SetImageColorSpace(ePdfColorSpace_DeviceGray);
@@ -453,6 +490,46 @@ void PdfImage::LoadFromTiff( const char* pszFilename )
             SetImageColorSpace(ePdfColorSpace_DeviceCMYK);
 		break;
 
+		case PHOTOMETRIC_PALETTE:
+		{
+			int numColors = (1 << bitsPixel);
+
+			PdfArray decode;
+			decode.insert( decode.end(), PdfVariant( static_cast<pdf_int64>(0) ) );
+			decode.insert( decode.end(), PdfVariant( static_cast<pdf_int64>(numColors-1) ) );
+			this->GetObject()->GetDictionary().AddKey( PdfName("Decode"), decode );
+
+			uint16 * rgbRed;
+			uint16 * rgbGreen;
+			uint16 * rgbBlue;
+			TIFFGetField(hInfile, TIFFTAG_COLORMAP, &rgbRed, &rgbGreen, &rgbBlue);
+
+			char *datap = new char[numColors*3];
+
+			for ( int clr = 0; clr < numColors; clr++ )
+			{
+				datap[3*clr+0] = rgbRed[clr]/257;
+				datap[3*clr+1] = rgbGreen[clr]/257;
+				datap[3*clr+2] = rgbBlue[clr]/257;
+			}
+		    PdfMemoryInputStream stream( datap, numColors*3 );
+
+		    // Create a colorspace object
+		    PdfObject* pIdxObject = this->GetObject()->GetOwner()->CreateObject();
+			pIdxObject->GetStream()->Set( &stream );
+    
+		    // Add the colorspace to our image
+		    PdfArray array;
+		    array.push_back( PdfName("Indexed") );
+		    array.push_back( PdfName("DeviceRGB") );
+			array.push_back( static_cast<pdf_int64>(numColors) );
+		    array.push_back( pIdxObject->Reference() );
+		    this->GetObject()->GetDictionary().AddKey( PdfName("ColorSpace"), array );
+
+			delete[] datap;
+		}
+		break;
+
 		default:
 	        TIFFClose(hInfile);
 	        PODOFO_RAISE_ERROR( ePdfError_UnsupportedImageFormat );
@@ -486,11 +563,117 @@ void PdfImage::LoadFromTiff( const char* pszFilename )
                  static_cast<unsigned int>(bitsPerSample), 
                  &stream);
 
-    delete buffer;
+    delete[] buffer;
 
     TIFFClose(hInfile);
 }
 #endif // PODOFO_HAVE_TIFF_LIB
+#ifdef PODOFO_HAVE_PNG_LIB
+void PdfImage::LoadFromPng( const char* pszFilename )
+{
+    if( !pszFilename )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+    FILE* hFile = fopen(pszFilename, "rb");
+    if( !hFile )
+    {
+        PODOFO_RAISE_ERROR_INFO( ePdfError_FileNotFound, pszFilename );
+    }
+
+    png_byte header[8];
+    fread(header, 1, 8, hFile);
+    if( png_sig_cmp(header, 0, 8) )
+    {
+        PODOFO_RAISE_ERROR_INFO( ePdfError_UnsupportedImageFormat, "The file could not be recognized as a PNG file." );
+    }
+    
+    png_structp pPng = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+    if( !pPng )
+    {
+        fclose( hFile );
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+    png_infop pInfo = png_create_info_struct(pPng);
+    if( !pInfo )
+    {
+        fclose( hFile );
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+    if( setjmp(png_jmpbuf(pPng)) )
+    {
+        fclose( hFile );
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+    png_init_io(pPng, hFile);
+    png_set_sig_bytes(pPng, 8);
+    png_read_info(pPng, pInfo);
+
+    int width = pInfo->width;
+    int height = pInfo->height;
+    png_read_update_info(pPng, pInfo);
+    
+    // Read the file
+    if( setjmp(png_jmpbuf(pPng)) ) 
+    {
+        fclose( hFile );
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+
+    long lLen = static_cast<long>(pInfo->rowbytes * height);
+    char* pBuffer = static_cast<char*>(malloc(sizeof(char) * lLen));
+    png_bytep pRows[height];
+    for(int y=0; y<height; y++)
+    {
+        pRows[y] = reinterpret_cast<png_bytep>(pBuffer + (y * pInfo->rowbytes));
+    }
+
+    png_read_image(pPng, pRows);
+    fclose(hFile);
+
+    m_rRect.SetWidth( width );
+    m_rRect.SetHeight( height );
+
+    switch( pInfo->channels )
+    {
+        case 3:
+            this->SetImageColorSpace( ePdfColorSpace_DeviceRGB );
+            break;
+        case 4:
+        {
+            this->SetImageColorSpace( ePdfColorSpace_DeviceCMYK );
+            // The jpeg-doc ist not specific in this point, but cmyk's seem to be stored
+            // in a inverted fashion. Fix by attaching a decode array
+            PdfArray decode;
+            decode.push_back( 1.0 );
+            decode.push_back( 0.0 );
+            decode.push_back( 1.0 );
+            decode.push_back( 0.0 );
+            decode.push_back( 1.0 );
+            decode.push_back( 0.0 );
+            decode.push_back( 1.0 );
+            decode.push_back( 0.0 );
+            
+            this->GetObject()->GetDictionary().AddKey( PdfName("Decode"), decode );
+        }
+        break;
+        default:
+            this->SetImageColorSpace( ePdfColorSpace_DeviceGray );
+            break;
+    }
+
+    // Set the image data and flate compress it
+    PdfMemoryInputStream stream( pBuffer, lLen );
+    this->SetImageData( width, height, pInfo->bit_depth, &stream );
+    
+    free(pBuffer);
+}
+#endif // PODOFO_HAVE_PNG_LIB
 
 const char* PdfImage::ColorspaceToName( EPdfColorSpace eColorSpace )
 {
@@ -502,6 +685,10 @@ const char* PdfImage::ColorspaceToName( EPdfColorSpace eColorSpace )
             return "DeviceRGB";
         case ePdfColorSpace_DeviceCMYK:
             return "DeviceCMYK";
+        case ePdfColorSpace_Separation:
+            return "Separation";
+        case ePdfColorSpace_CieLab:
+            return "Lab";
         default:
             return NULL;
     }
