@@ -308,6 +308,11 @@ class PODOFO_API PdfMemDocument : public PdfDocument {
      */
     PdfObject* GetMetadata() const { return GetNamedObjectFromCatalog( "Metadata" ); }
 
+    /** Get access to the MarkInfo dictionary (ISO 32000-1:2008 14.7.1)
+     *  \returns PdfObject the MarkInfo dictionary
+     */
+    PdfObject* GetMarkInfo() const { return GetNamedObjectFromCatalog( "MarkInfo" ); }
+
     /** Creates a PdfFont object from an existing font.
      *
      *  \param pObject a PdfObject that is a font
@@ -326,6 +331,10 @@ class PODOFO_API PdfMemDocument : public PdfDocument {
     const PdfMemDocument & InsertPages( const PdfMemDocument & rDoc, int inFirstPage, int inNumPages );
 
     /** Deletes one or more pages from this document
+     *  It does NOT remove any PdfObjects from memory - just the reference from the pages tree.
+     *  If you want to delete resources of this page, you have to delete them yourself,
+     *  but the resources might be used by other pages, too.
+     *
      *  \param inFirstPage the first page number to delete (0-based)
      *  \param inNumPages the number of pages to delete
      *  \returns this document
