@@ -29,6 +29,8 @@
 #include "PdfParserObject.h"
 #include "PdfStream.h"
 #include "PdfVariant.h"
+#include "PdfDefinesPrivate.h"
+#include "PdfXRefStreamParserObject.h"
 
 #include <cstring>
 #include <cstdlib>
@@ -107,6 +109,7 @@ void PdfParser::Init()
     m_pTrailer        = NULL;
     m_pLinearization  = NULL;
     m_offsets.clear();
+
     m_pEncrypt        = NULL;
 
     m_ePdfVersion     = ePdfVersion_Default;
@@ -115,6 +118,10 @@ void PdfParser::Init()
     m_nFirstObject    = 0;
     m_nNumObjects     = 0;
     m_nXRefLinearizedOffset = 0;
+
+    m_bStrictParsing  = false;
+    m_bIgnoreBrokenObjects = false;
+    m_nIncrementalUpdates = 0;
 }
 
 void PdfParser::ParseFile( const char* pszFilename, bool bLoadOnDemand )
@@ -174,7 +181,6 @@ void PdfParser::ParseFile( const char* pBuffer, long lLen, bool bLoadOnDemand )
 
 void PdfParser::ParseFile( const PdfRefCountedInputDevice & rDevice, bool bLoadOnDemand )
 {
-    // make sure everything is clean
     Clear();
 
     m_device = rDevice;
@@ -185,19 +191,27 @@ void PdfParser::ParseFile( const PdfRefCountedInputDevice & rDevice, bool bLoadO
     {
         PODOFO_RAISE_ERROR( ePdfError_NoPdfFile );
     }
-
+    
     ReadDocumentStructure();
     try {
         ReadObjects();
     } catch( PdfError & e ) {
+        if( e.GetError() == ePdfError_InvalidPassword ) 
+        {
+            // Do not clean up, expect user to call ParseFile again
+            throw e;
+        }
+
+        // If this is being called from a constructor then the
+        // destructor will not be called.
+        // Clean up here  
+        Clear();
         e.AddToCallstack( __FILE__, __LINE__, "Unable to load objects from file." );
         throw e;
     }
-
-    // Now sort the list of objects
-    m_vecObjects->Sort();
 }
 
+
 void PdfParser::Clear()
 {
     m_setObjectStreams.clear();
@@ -219,7 +233,8 @@ void PdfParser::Clear()
 
 void PdfParser::ReadDocumentStructure()
 {
-    HasLinearizationDict();
+// Ulrich Arnold 8.9.2009, deactivated because of problems during reading xref's
+//    HasLinearizationDict();
     
     // position at the end of the file to search the xref table.
     m_device.Device()->Seek( 0, std::ios_base::end );
@@ -259,7 +274,7 @@ void PdfParser::ReadDocumentStructure()
 
     if( m_pTrailer->IsDictionary() && m_pTrailer->GetDictionary().HasKey( PdfName::KeySize ) )
     {
-        m_nNumObjects = m_pTrailer->GetDictionary().GetKeyAsLong( PdfName::KeySize );
+        m_nNumObjects = static_cast<long>(m_pTrailer->GetDictionary().GetKeyAsLong( PdfName::KeySize ));
     }
     else
     {
@@ -288,15 +303,6 @@ void PdfParser::ReadDocumentStructure()
         throw e;
     }
 
-    if( m_pTrailer->GetDictionary().HasKey( "Prev" ) )
-    {
-        try {
-            ReadXRefContents( m_pTrailer->GetDictionary().GetKeyAsLong( "Prev", 0 ) ); 
-        } catch( PdfError & e ) {
-            e.AddToCallstack( __FILE__, __LINE__, "Unable to load /Prev xref entries." );
-            throw e;
-        }
-    }
 }
 
 bool PdfParser::IsPdfFile()
@@ -307,7 +313,7 @@ bool PdfParser::IsPdfFile()
     if( m_device.Device()->Read( m_buffer.GetBuffer(), PDF_MAGIC_LEN ) != PDF_MAGIC_LEN )
         return false;
 
-     if( strncmp( m_buffer.GetBuffer(), szPdfMagicStart, strlen( szPdfMagicStart ) ) != 0 )
+    if( strncmp( m_buffer.GetBuffer(), szPdfMagicStart, strlen( szPdfMagicStart ) ) != 0 )
         return false;
         
     // try to determine the excact PDF version of the file
@@ -330,8 +336,21 @@ void PdfParser::HasLinearizationDict()
     }
 
     m_device.Device()->Seek( 0 );
-    // look for a linearization dictionary in the first 1024 bytes
-    if( m_device.Device()->Read( m_buffer.GetBuffer(), m_buffer.GetSize() ) != m_buffer.GetSize() )
+
+    // The linearization dictionary must be in the first 1024 
+    // bytes of the PDF, our buffer might be larger so.
+    // Therefore read only the first 1024 byte.
+    // Normally we should jump to the end of the file, to determine
+    // it's filesize and read the min(1024, filesize) to not fail
+    // on smaller files, but jumping to the end is against the idea
+    // of linearized PDF. Therefore just check if we read anything.
+    const std::streamoff MAX_READ = 1024;
+    PdfRefCountedBuffer linearizeBuffer( MAX_READ );
+
+    std::streamoff size = m_device.Device()->Read( linearizeBuffer.GetBuffer(), 
+                                                   linearizeBuffer.GetSize() );
+    // Only fail if we read nothing, to allow files smaller than MAX_READ
+    if( static_cast<size_t>(size) <= 0 )
     {
         // Clear the error state from the bad read
         m_device.Device()->Clear();
@@ -348,7 +367,8 @@ void PdfParser::HasLinearizationDict()
     while( *pszObj && (PdfTokenizer::IsWhitespace( *pszObj ) || (*pszObj >= '0' && *pszObj <= '9')) )
         --pszObj;
 
-    m_pLinearization = new PdfParserObject( m_vecObjects, m_device, m_buffer, pszObj - m_buffer.GetBuffer() + 2 );
+    m_pLinearization = new PdfParserObject( m_vecObjects, m_device, linearizeBuffer,
+                                            pszObj - linearizeBuffer.GetBuffer() + 2 );
 
     try {
         // Do not care for encryption here, as the linearization dictionary does not contain strings or streams
@@ -368,7 +388,7 @@ void PdfParser::HasLinearizationDict()
         return;
     }
     
-    long      lXRef      = -1;
+    long long      lXRef      = -1;
     lXRef = m_pLinearization->GetDictionary().GetKeyAsLong( "T", lXRef );
     if( lXRef == -1 )
     {
@@ -376,7 +396,7 @@ void PdfParser::HasLinearizationDict()
     }
 
     // avoid moving to a negative file position here
-    m_device.Device()->Seek( (lXRef-PDF_XREF_BUF > 0 ? lXRef-PDF_XREF_BUF : PDF_XREF_BUF) );
+    m_device.Device()->Seek( (static_cast<pdf_long>(lXRef-PDF_XREF_BUF) > 0 ? static_cast<pdf_long>(lXRef-PDF_XREF_BUF) : PDF_XREF_BUF) );
     m_nXRefLinearizedOffset = m_device.Device()->Tell();
 
     if( m_device.Device()->Read( m_buffer.GetBuffer(), PDF_XREF_BUF ) != PDF_XREF_BUF )
@@ -411,7 +431,7 @@ void PdfParser::HasLinearizationDict()
         }
 
         {
-            m_nXRefLinearizedOffset = lXRef;
+            m_nXRefLinearizedOffset = static_cast<pdf_long>(lXRef);
             /*
             eCode = ReadXRefStreamContents();
             i     = 0;
@@ -465,8 +485,12 @@ void PdfParser::ReadNextTrailer()
         
         if( trailer.GetDictionary().HasKey( "Prev" ) )
         {
+            // Whenever we read a Prev key, 
+            // we know that the file was updated.
+            m_nIncrementalUpdates++;
+
             try {
-                ReadXRefContents( trailer.GetDictionary().GetKeyAsLong( "Prev", 0 ) );
+                ReadXRefContents( static_cast<pdf_long>(trailer.GetDictionary().GetKeyAsLong( "Prev", 0 )) );
             } catch( PdfError & e ) {
                 e.AddToCallstack( __FILE__, __LINE__, "Unable to load /Prev xref entries." );
                 throw e;
@@ -485,7 +509,9 @@ void PdfParser::ReadTrailer()
     
     if( !this->IsNextToken( "trailer" ) ) 
     {
-        if( m_ePdfVersion < ePdfVersion_1_5 )
+//      if( m_ePdfVersion < ePdfVersion_1_5 )
+//		Ulrich Arnold 19.10.2009, found linearized 1.3-pdf's with trailer-info in xref-stream
+        if( m_ePdfVersion < ePdfVersion_1_3 )
         {
             PODOFO_RAISE_ERROR( ePdfError_NoTrailer );
         }
@@ -517,7 +543,7 @@ void PdfParser::ReadTrailer()
     }
 }
 
-void PdfParser::ReadXRef( long* pXRefOffset )
+void PdfParser::ReadXRef( pdf_long* pXRefOffset )
 {
     FindToken( "startxref", PDF_XREF_BUF );
 
@@ -529,16 +555,38 @@ void PdfParser::ReadXRef( long* pXRefOffset )
     *pXRefOffset = this->GetNextNumber();
 }
 
-void PdfParser::ReadXRefContents( long lOffset, bool bPositionAtEnd )
+void PdfParser::ReadXRefContents( pdf_long lOffset, bool bPositionAtEnd )
 {
-    long        nFirstObject = 0;
-    long        nNumObjects  = 0;
+    long long nFirstObject = 0;
+    long long nNumObjects  = 0;
 
-    m_device.Device()->Seek( lOffset );
+    size_t curPosition = m_device.Device()->Tell();
+    m_device.Device()->Seek(0,std::ios_base::end);
+    std::streamoff fileSize = m_device.Device()->Tell();
+    m_device.Device()->Seek(curPosition,std::ios_base::beg);
+
+    if (lOffset > fileSize)
+    { 
+        // Invalid "startxref" Peter Petrov 23 December 2008
+        FindToken( "startxref", PDF_XREF_BUF );
+        lOffset = m_device.Device()->Tell();
+        // TODO: hard coded value "4"
+        m_buffer.Resize(PDF_XREF_BUF*4);
+        FindToken2("xref", PDF_XREF_BUF*4,lOffset);
+        m_buffer.Resize(PDF_XREF_BUF);
+        lOffset = m_device.Device()->Tell();
+        m_nXRefOffset = lOffset;
+    }
+    else
+    {
+        m_device.Device()->Seek( lOffset );
+    }
     
     if( !this->IsNextToken( "xref" ) )
     {
-        if( m_ePdfVersion < ePdfVersion_1_5 )
+//      if( m_ePdfVersion < ePdfVersion_1_5 )
+//		Ulrich Arnold 19.10.2009, found linearized 1.3-pdf's with trailer-info in xref-stream
+        if( m_ePdfVersion < ePdfVersion_1_3 )
         {
             PODOFO_RAISE_ERROR( ePdfError_NoXRef );
         }
@@ -557,16 +605,21 @@ void PdfParser::ReadXRefContents( long lOffset, bool bPositionAtEnd )
             nNumObjects  = this->GetNextNumber();
 
 #ifdef PODOFO_VERBOSE_DEBUG
-            PdfError::DebugMessage("Reading numbers: %i %i\n", nFirstObject, nNumObjects );
+            PdfError::DebugMessage("Reading numbers: %lli %lli\n", nFirstObject, nNumObjects );
 #endif // PODOFO_VERBOSE_DEBUG
 
             if( bPositionAtEnd )
             {
+#ifdef _WIN32
+				m_device.Device()->Seek( static_cast<std::streamoff>(nNumObjects* PDF_XREF_ENTRY_SIZE), std::ios_base::cur );
+#else
                 m_device.Device()->Seek( nNumObjects* PDF_XREF_ENTRY_SIZE, std::ios_base::cur );
-            }
+#endif // _WIN32
+			}
             else
+            {
                 ReadXRefSubsection( nFirstObject, nNumObjects );
-
+            }
         } catch( PdfError & e ) {
             if( e == ePdfError_NoNumber || e == ePdfError_InvalidXRef || e == ePdfError_UnexpectedEOF ) 
                 break;
@@ -589,12 +642,12 @@ void PdfParser::ReadXRefContents( long lOffset, bool bPositionAtEnd )
     }
 }
 
-void PdfParser::ReadXRefSubsection( long & nFirstObject, long & nNumObjects )
+void PdfParser::ReadXRefSubsection( long long & nFirstObject, long long & nNumObjects )
 {
-    int         count        = 0;
+    int count = 0;
 
 #ifdef PODOFO_VERBOSE_DEBUG
-    PdfError::DebugMessage("Reading XRef Section: %i with %i Objects.\n", nFirstObject, nNumObjects );
+    PdfError::DebugMessage("Reading XRef Section: %lli with %lli Objects.\n", nFirstObject, nNumObjects );
 #endif // PODOFO_VERBOSE_DEBUG 
 
     if ( nFirstObject + nNumObjects > m_nNumObjects )
@@ -602,26 +655,55 @@ void PdfParser::ReadXRefSubsection( long & nFirstObject, long & nNumObjects )
         // Total number of xref entries to read is greater than the /Size
         // specified in the trailer if any. That's an error unless we're trying
         // to recover from a missing /Size entry.
-	PdfError::LogMessage( eLogSeverity_Warning,
-			      "There are more objects (%i) in this XRef table than "
-			      "specified in the size key of the trailer directory (%i)!\n",
+		PdfError::LogMessage( eLogSeverity_Warning,
+			      "There are more objects (%lli) in this XRef table than "
+			      "specified in the size key of the trailer directory (%lli)!\n",
 			      nFirstObject + nNumObjects, m_nNumObjects );
 
-	m_nNumObjects = nFirstObject + nNumObjects;
-	m_offsets.resize(nFirstObject+nNumObjects);
+#ifdef _WIN32
+		m_nNumObjects = static_cast<long>(nFirstObject + nNumObjects);
+		m_offsets.resize(static_cast<long>(nFirstObject+nNumObjects));
+#else
+		m_nNumObjects = nFirstObject + nNumObjects;
+		m_offsets.resize(nFirstObject+nNumObjects);
+#endif // _WIN32
+	}
+
+    // consume all whitespaces
+    int charcode;
+    while( this->IsWhitespace((charcode = m_device.Device()->Look())) )
+    {
+        m_device.Device()->GetChar();
     }
 
     while( count < nNumObjects && m_device.Device()->Read( m_buffer.GetBuffer(), PDF_XREF_ENTRY_SIZE ) == PDF_XREF_ENTRY_SIZE )
     {
+        char empty1;
+        char empty2;
         m_buffer.GetBuffer()[PDF_XREF_ENTRY_SIZE] = '\0';
 
-        const int objID = nFirstObject+count;
+#ifdef _WIN32
+		const int objID = static_cast<int>(nFirstObject+count);
+#else
+		const int objID = nFirstObject+count;
+#endif // _WIN32
+
         if( !m_offsets[objID].bParsed )
         {
             m_offsets[objID].bParsed = true;
-            sscanf( m_buffer.GetBuffer(), "%10ld %5ld %c \n", 
+#ifdef _WIN64
+            sscanf( m_buffer.GetBuffer(), "%10I64d %5ld %c%c%c", 
                     &(m_offsets[objID].lOffset), 
-                    &(m_offsets[objID].lGeneration), &(m_offsets[objID].cUsed) );
+                    &(m_offsets[objID].lGeneration), &(m_offsets[objID].cUsed), &empty1, &empty2 );
+#else
+            long long int tmp1;
+            long int tmp2;
+            sscanf( m_buffer.GetBuffer(), "%10lld %5ld %c%c%c", 
+                    &tmp1, &tmp2, &(m_offsets[objID].cUsed), &empty1, &empty2 );
+
+            m_offsets[objID].lOffset = tmp1;
+            m_offsets[objID].lGeneration = tmp2;
+#endif
        }
 
         ++count;
@@ -629,40 +711,18 @@ void PdfParser::ReadXRefSubsection( long & nFirstObject, long & nNumObjects )
 
     if( count != nNumObjects )
     {
-        PdfError::LogMessage( eLogSeverity_Warning, "Count of readobject is %i. Expected %i.\n", count, nNumObjects );
+        PdfError::LogMessage( eLogSeverity_Warning, "Count of readobject is %i. Expected %lli.\n", count, nNumObjects );
         PODOFO_RAISE_ERROR( ePdfError_NoXRef );
     }
 
 }
 
-void PdfParser::ReadXRefStreamContents( long lOffset, bool bReadOnlyTrailer )
+void PdfParser::ReadXRefStreamContents( pdf_long lOffset, bool bReadOnlyTrailer )
 {
-    char*       pBuffer;
-    char*       pStart;
-    long        lBufferLen;
-    long        lSize     = 0;
-    PdfVariant  vWArray;
-    PdfObject*  pObj;
-
-    long        nW[W_ARRAY_SIZE] = { 0, 0, 0 };
-    int         i;
-
     m_device.Device()->Seek( lOffset );
 
-    PdfParserObject xrefObject( m_vecObjects, m_device, m_buffer );
-    // Ignore the encryption in the XREF as the XREF stream must no be encrypted (see PDF Reference 3.4.7)
-    xrefObject.ParseFile( NULL );
-
-    if( !xrefObject.GetDictionary().HasKey( PdfName::KeyType ) )
-    {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-    } 
-
-    pObj = xrefObject.GetDictionary().GetKey( PdfName::KeyType );
-    if( !pObj->IsName() || ( pObj->GetName() != "XRef" ) )
-    {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-    } 
+    PdfXRefStreamParserObject xrefObject( m_vecObjects, m_device, m_buffer, &m_offsets );
+    xrefObject.Parse();
 
     if( !m_pTrailer )
         m_pTrailer = new PdfParserObject( m_vecObjects, m_device, m_buffer );
@@ -672,142 +732,13 @@ void PdfParser::ReadXRefStreamContents( long lOffset, bool bReadOnlyTrailer )
     if( bReadOnlyTrailer )
         return;
 
-    if( !xrefObject.GetDictionary().HasKey( PdfName::KeySize ) || !xrefObject.GetDictionary().HasKey( "W" ) )
+    xrefObject.ReadXRefTable();
+
+    // Check for a previous XRef stream
+    if(xrefObject.HasPrevious()) 
     {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-    }
-
-    lSize   = xrefObject.GetDictionary().GetKeyAsLong( PdfName::KeySize, 0 );
-    vWArray = *(xrefObject.GetDictionary().GetKey( "W" ));
-
-    // The pdf reference states that W is always an array with 3 entries
-    // all of them have to be integeres
-    if( !vWArray.IsArray() || vWArray.GetArray().size() != 3 )
-    {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-    }
-
-    for( i=0;i<W_ARRAY_SIZE;i++ )
-    {
-        if( !vWArray.GetArray()[i].IsNumber() )
-        {
-            PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-        }
-
-        nW[i] = vWArray.GetArray()[i].GetNumber();
-    }
-
-    std::vector<int> vecIndeces;
-    // get the first object number in this crossref stream.
-    // it is not required to have an index key though.
-    if( xrefObject.GetDictionary().HasKey( "Index" ) )
-    {
-        // reuse vWArray!!
-        vWArray = *(xrefObject.GetDictionary().GetKey( "Index" ));
-        if( !vWArray.IsArray() )
-        {
-            PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-        }
-
-        TCIVariantList it = vWArray.GetArray().begin();
-        while ( it != vWArray.GetArray().end() )
-        {
-            vecIndeces.push_back( (*it).GetNumber() );
-            ++it;
-        }
-    }
-    else
-    {
-        vecIndeces.push_back( 0 );
-        vecIndeces.push_back( lSize );
-    }
-
-    if( vecIndeces.size() % 2 )
-    {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-    }
-
-    if( !xrefObject.HasStreamToParse() )
-    {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
-    }
-
-    xrefObject.GetStream()->GetFilteredCopy( &pBuffer, &lBufferLen );
-
-    pStart        = pBuffer;
-    int nCurIndex = 0;
-    while( nCurIndex < static_cast<int>(vecIndeces.size()) && pBuffer - pStart < lBufferLen )
-    {
-        long nFirstObj = vecIndeces[nCurIndex];
-        long nCount    = vecIndeces[nCurIndex+1];
-
-        PdfError::DebugMessage( "Reading Subrefsection: %li %li\n", nFirstObj, nCount );
-
-        while( nCount-- && pBuffer - pStart < lBufferLen ) 
-        {
-            ReadXRefStreamEntry( pBuffer, lBufferLen, nW, nFirstObj++ );
-            pBuffer += (nW[0] + nW[1] + nW[2]);
-        }
-
-        nCurIndex += 2;
-    }
-    free( pStart );
-
-    if( xrefObject.GetDictionary().HasKey("Prev") )
-    {
-        lOffset = xrefObject.GetDictionary().GetKeyAsLong( "Prev", 0 );
-        ReadXRefStreamContents( lOffset, bReadOnlyTrailer );
-    }
-}
-
-void PdfParser::ReadXRefStreamEntry( char* pBuffer, long, long lW[W_ARRAY_SIZE], int nObjNo )
-{
-    int              i, z;
-    unsigned long    nData[W_ARRAY_SIZE];
-
-    for( i=0;i<W_ARRAY_SIZE;i++ )
-    {
-        if( lW[i] > W_MAX_BYTES )
-        {
-            PdfError::LogMessage( eLogSeverity_Error, "The XRef stream dictionary has an entry in /W of size %i.\nThe maximum supported value is %i.\n", lW[i], W_MAX_BYTES );
-
-            PODOFO_RAISE_ERROR( ePdfError_InvalidXRefStream );
-        }
-        
-        nData[i] = 0;
-        for( z=W_MAX_BYTES-lW[i];z<W_MAX_BYTES;z++ )
-        {
-            nData[i] = (nData[i] << 8) + static_cast<unsigned char>(*pBuffer);
-            ++pBuffer;
-        }
-    }
-
-    m_offsets[nObjNo].bParsed = true;
-
-    switch( nData[0] ) // nData[0] contains the type information of this entry
-    {
-        case 0:
-            // a free object
-            m_offsets[nObjNo].lOffset     = nData[1];
-            m_offsets[nObjNo].lGeneration = nData[2];
-            m_offsets[nObjNo].cUsed       = 'f';
-            break;
-        case 1:
-            // normal uncompressed object
-            m_offsets[nObjNo].lOffset     = nData[1];
-            m_offsets[nObjNo].lGeneration = nData[2];
-            m_offsets[nObjNo].cUsed       = 'n';
-            break;
-        case 2:
-            // object that is part of an object stream
-            m_offsets[nObjNo].lOffset     = nData[2]; // index in the object stream
-            m_offsets[nObjNo].lGeneration = nData[1]; // object number of the stream
-            m_offsets[nObjNo].cUsed       = 's';      // mark as stream
-            break;
-        default:
-        {
-            PODOFO_RAISE_ERROR( ePdfError_InvalidXRefType );
-        }
+        m_nIncrementalUpdates++;
+        this->ReadXRefStreamContents( xrefObject.GetPreviousOffset(), bReadOnlyTrailer );
     }
 }
 
@@ -848,7 +779,8 @@ bool PdfParser::QuickEncryptedCheck( const char* pszFilename )
 
         // Check for encryption and make sure that the encryption object
         // is loaded before all other objects
-        if( m_pTrailer->GetDictionary().HasKey( PdfName("Encrypt") ) )
+        const PdfObject * encObj = m_pTrailer->GetDictionary().GetKey( PdfName("Encrypt") );
+        if( encObj && ! encObj->IsNull() ) 
         {
             bEncryptStatus = true;
         }
@@ -872,24 +804,26 @@ void PdfParser::ReadObjects()
 
     // Check for encryption and make sure that the encryption object
     // is loaded before all other objects
-    if( m_pTrailer->GetDictionary().HasKey( PdfName("Encrypt") ) )
+    PdfObject* pEncrypt = m_pTrailer->GetDictionary().GetKey( PdfName("Encrypt") );
+    if( pEncrypt && !pEncrypt->IsNull() )
     {
 #ifdef PODOFO_VERBOSE_DEBUG
         PdfError::DebugMessage("The PDF file is encrypted.\n" );
 #endif // PODOFO_VERBOSE_DEBUG
 
-        PdfObject* pEncrypt = m_pTrailer->GetDictionary().GetKey( PdfName("Encrypt") );
         if( pEncrypt->IsReference() ) 
         {
             i = pEncrypt->GetReference().ObjectNumber();
-
             pObject = new PdfParserObject( m_vecObjects, m_device, m_buffer, m_offsets[i].lOffset );
-            pObject->SetLoadOnDemand( m_bLoadOnDemand );
+            pObject->SetLoadOnDemand( false ); // Never load this on demand, as we will use it immediately
             try {
                 pObject->ParseFile( NULL ); // The encryption dictionary is not encrypted :)
-                m_vecObjects->push_back( pObject );
+                // Never add the encryption dictionary to m_vecObjects
+                // we create a new one, if we need it for writing
+                // m_vecObjects->push_back( pObject );
                 m_offsets[i].bParsed = false;
                 m_pEncrypt = PdfEncrypt::CreatePdfEncrypt( pObject );
+                delete pObject;
             } catch( PdfError & e ) {
                 std::ostringstream oss;
                 if( pObject )
@@ -901,10 +835,13 @@ void PdfParser::ReadObjects()
 
                 e.AddToCallstack( __FILE__, __LINE__, oss.str().c_str() );
                 throw e;
+
             }
         }
         else if( pEncrypt->IsDictionary() ) 
+        {
             m_pEncrypt = PdfEncrypt::CreatePdfEncrypt( pEncrypt );
+        }
         else
         {
             PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidEncryptionDict, 
@@ -938,14 +875,28 @@ void PdfParser::ReadObjectsInternal()
     // Read objects
     for( i=0; i < m_nNumObjects; i++ )
     {
-        if( m_offsets[i].bParsed && m_offsets[i].cUsed == 'n' )
+        if( m_offsets[i].bParsed && m_offsets[i].cUsed == 'n' && m_offsets[i].lOffset > 0 )
         {
+            //printf("Reading object %i 0 R from %li\n", i, m_offsets[i].lOffset );
+            
             pObject = new PdfParserObject( m_vecObjects, m_device, m_buffer, m_offsets[i].lOffset );
             pObject->SetLoadOnDemand( m_bLoadOnDemand );
             try {
                 pObject->ParseFile( m_pEncrypt );
                 nLast = pObject->Reference().ObjectNumber();
 
+                /*
+                if( i != pObject->Reference().ObjectNumber() ) 
+                {
+                    printf("Expected %i got %i\n", i, pObject->Reference().ObjectNumber());
+                }
+                if( pObject->Reference().ObjectNumber() != i ) 
+                {
+                    printf("EXPECTED: %i got %i\n", i, pObject->Reference().ObjectNumber() );
+                    abort();
+                }
+                */
+
                 // final pdf should not contain a linerization dictionary as it contents are invalid 
                 // as we change some objects and the final xref table
                 if( m_pLinearization && nLast == static_cast<int>(m_pLinearization->Reference().ObjectNumber()) )
@@ -959,18 +910,56 @@ void PdfParser::ReadObjectsInternal()
                 std::ostringstream oss;
                 if( pObject )
                 {
-                    oss << "Error while loading object " << pObject->Reference().ObjectNumber() << " " 
-                        << pObject->Reference().GenerationNumber() << std::endl;
+                    oss << "Error while loading object " << pObject->Reference().ObjectNumber() 
+                        << " " << pObject->Reference().GenerationNumber() 
+                        << " Offset = " << m_offsets[i].lOffset
+                        << " Index = " << i << std::endl;
                     delete pObject;
                 }
 
-                e.AddToCallstack( __FILE__, __LINE__, oss.str().c_str() );
-                throw e;
+                if( m_bIgnoreBrokenObjects ) 
+                {
+                    PdfError::LogMessage( eLogSeverity_Error, oss.str().c_str() );
+                    m_vecObjects->AddFreeObject( PdfReference( i, 0 ) );
+                }
+                else
+                {
+                    e.AddToCallstack( __FILE__, __LINE__, oss.str().c_str() );
+                    throw e;
+                }
             }
         }
-        else if( m_offsets[i].bParsed && m_offsets[i].cUsed == 'f' && m_offsets[i].lOffset )
+        else if( m_offsets[i].bParsed && m_offsets[i].cUsed == 'n' && (m_offsets[i].lOffset == 0)  )
         {
-            m_vecObjects->AddFreeObject( PdfReference( m_offsets[i].lOffset, 1 ) ); // TODO: do not hard code
+            // There are broken PDFs which add objects with 'n' 
+            // and 0 offset and 0 generation number
+            // to the xref table instead of using free objects
+            // treating them as free objects
+            if( m_bStrictParsing ) 
+            {
+                PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidXRef,
+                                         "Found object with 0 offset which should be 'f' instead of 'n'." );
+            }
+            else
+            {
+                PdfError::LogMessage( eLogSeverity_Warning, 
+                                      "Treating object %i 0 R as a free object." );
+                m_vecObjects->AddFreeObject( PdfReference( i, 1LL ) );
+            }
+        }
+// Ulrich Arnold 30.7.2009: the linked free list in the xref section is not always correct in pdf's
+//							(especially Illustrator) but Acrobat still accepts them. I've seen XRefs 
+//							where some object-numbers are alltogether missing and multiple XRefs where 
+//							the link list is broken.
+//							Because PdfVecObjects relies on a unbroken range, fill the free list more
+//							robustly from all places which are either free or unparsed
+//      else if( m_offsets[i].bParsed && m_offsets[i].cUsed == 'f' && m_offsets[i].lOffset )
+//      {
+//          m_vecObjects->AddFreeObject( PdfReference( static_cast<int>(m_offsets[i].lOffset), 1LL ) ); // TODO: do not hard code
+//      }
+        else if( (!m_offsets[i].bParsed || m_offsets[i].cUsed == 'f') && i != 0 )
+        {
+			m_vecObjects->AddFreeObject( PdfReference( static_cast<int>(i), 1LL ) ); // TODO: do not hard code generation number
         }
     }
 
@@ -985,9 +974,10 @@ void PdfParser::ReadObjectsInternal()
         if( m_offsets[i].bParsed && m_offsets[i].cUsed == 's' ) // we have an object stream
         {
 #if defined(PODOFO_VERBOSE_DEBUG)
-            if (m_bLoadOnDemand) cerr << "Demand loading on, but can't demand-load found object stream." << endl;
+            if (m_bLoadOnDemand) cerr << "Demand loading on, but can't demand-load from object stream." << endl;
 #endif
-            ReadObjectFromStream( m_offsets[i].lGeneration, m_offsets[i].lOffset );
+            ReadObjectFromStream( static_cast<int>(m_offsets[i].lGeneration), 
+                                  static_cast<int>(m_offsets[i].lOffset) );
         }
     }
 
@@ -1008,6 +998,12 @@ void PdfParser::ReadObjectsInternal()
                 pObject->GetStream();
         }
     }
+
+
+    // Now sort the list of objects
+    m_vecObjects->Sort();
+
+    UpdateDocumentVersion();
 }
 
 void PdfParser::SetPassword( const std::string & sPassword )
@@ -1050,11 +1046,11 @@ void PdfParser::ReadObjectFromStream( int nObjNo, int )
         PODOFO_RAISE_ERROR_INFO( ePdfError_NoObject, oss.str().c_str() );
     }
     
-    long lNum   = pStream->GetDictionary().GetKeyAsLong( "N", 0 );
-    long lFirst = pStream->GetDictionary().GetKeyAsLong( "First", 0 );
+    long long lNum   = pStream->GetDictionary().GetKeyAsLong( "N", 0 );
+    long long lFirst = pStream->GetDictionary().GetKeyAsLong( "First", 0 );
     
     char * pBuffer;
-    long lBufferLen;
+    pdf_long lBufferLen;
 
     pStream->GetStream()->GetFilteredCopy( &pBuffer, &lBufferLen );
 
@@ -1066,17 +1062,17 @@ void PdfParser::ReadObjectFromStream( int nObjNo, int )
     PdfVariant               var;
     int                      i = 0;
 
-    while( i < lNum )
+    while( static_cast<long long>(i) < lNum )
     {
-        const long lObj          = tokenizer.GetNextNumber();
-        const long lOff          = tokenizer.GetNextNumber();
+        const long long lObj     = tokenizer.GetNextNumber();
+        const long long lOff     = tokenizer.GetNextNumber();
         const std::streamoff pos = device.Device()->Tell();
 
         // move to the position of the object in the stream
-        device.Device()->Seek( lFirst + lOff );
+        device.Device()->Seek( static_cast<std::streamoff>(lFirst + lOff) );
 
         tokenizer.GetNextVariant( var, m_pEncrypt );
-        m_vecObjects->push_back( new PdfObject( PdfReference( lObj, 0 ), var ) );
+        m_vecObjects->push_back( new PdfObject( PdfReference( static_cast<int>(lObj), 0LL ), var ) );
 
         // move back to the position inside of the table of contents
         device.Device()->Seek( pos );
@@ -1104,9 +1100,52 @@ void PdfParser::FindToken( const char* pszToken, const long lRange )
                 "Failed to seek to EOF when looking for xref");
     }
 
-    long           lXRefBuf  = PDF_MIN( nFileSize, lRange );
-    const int      nTokenLen = strlen( pszToken );
-    int            i;
+    pdf_long lXRefBuf  = PDF_MIN( static_cast<pdf_long>(nFileSize), static_cast<pdf_long>(lRange) );
+    size_t   nTokenLen = strlen( pszToken );
+
+    m_device.Device()->Seek( -lXRefBuf, std::ios_base::cur );
+    if( m_device.Device()->Read( m_buffer.GetBuffer(), lXRefBuf ) != lXRefBuf && !m_device.Device()->Eof() )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
+    }
+
+    m_buffer.GetBuffer()[lXRefBuf] = '\0';
+
+    int i; // Do not make this unsigned, this will cause infinte loops in files without trailer
+ 
+    // search backwards in the buffer in case the buffer contains null bytes
+    // because it is right after a stream (can't use strstr for this reason)
+    for( i = lXRefBuf - nTokenLen; i >= 0; i-- )
+    {
+        if( strncmp( m_buffer.GetBuffer()+i, pszToken, nTokenLen ) == 0 )
+        {
+            break;
+        }
+    }
+
+    if( !i )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InternalLogic );
+    }
+
+    m_device.Device()->Seek( (lXRefBuf-i)*-1, std::ios_base::end );
+}
+
+// Peter Petrov 23 December 2008
+void PdfParser::FindToken2( const char* pszToken, const long lRange, size_t searchEnd )
+{
+    m_device.Device()->Seek( searchEnd, std::ios_base::beg );
+
+    std::streamoff nFileSize = m_device.Device()->Tell();
+    if (nFileSize == -1)
+    {
+        PODOFO_RAISE_ERROR_INFO(
+                ePdfError_NoXRef,
+                "Failed to seek to EOF when looking for xref");
+    }
+
+    pdf_long      lXRefBuf  = PDF_MIN( static_cast<pdf_long>(nFileSize), static_cast<pdf_long>(lRange) );
+    size_t        nTokenLen = strlen( pszToken );
 
     m_device.Device()->Seek( -lXRefBuf, std::ios_base::cur );
     if( m_device.Device()->Read( m_buffer.GetBuffer(), lXRefBuf ) != lXRefBuf && !m_device.Device()->Eof() )
@@ -1118,6 +1157,7 @@ void PdfParser::FindToken( const char* pszToken, const long lRange )
 
     // search backwards in the buffer in case the buffer contains null bytes
     // because it is right after a stream (can't use strstr for this reason)
+    int i; // Do not use an unsigned variable here
     for( i = lXRefBuf - nTokenLen; i >= 0; i-- )
         if( strncmp( m_buffer.GetBuffer()+i, pszToken, nTokenLen ) == 0 )
         {
@@ -1129,7 +1169,7 @@ void PdfParser::FindToken( const char* pszToken, const long lRange )
         PODOFO_RAISE_ERROR( ePdfError_InternalLogic );
     }
 
-    m_device.Device()->Seek( (lXRefBuf-i)*-1, std::ios_base::end );
+    m_device.Device()->Seek( searchEnd + (lXRefBuf-i)*-1, std::ios_base::beg );
 }
 
 const PdfString & PdfParser::GetDocumentId() 
@@ -1142,6 +1182,38 @@ const PdfString & PdfParser::GetDocumentId()
     return m_pTrailer->GetDictionary().GetKey( PdfName("ID") )->GetArray()[0].GetString();
 }
 
+void PdfParser::UpdateDocumentVersion()
+{
+    if( m_pTrailer->IsDictionary() && m_pTrailer->GetDictionary().HasKey( PdfName("Root") ) )
+    {
+        PdfObject* pCatalog = m_pTrailer->GetDictionary().GetKey( PdfName("Root") );
+        if( pCatalog->IsReference() ) 
+        {
+            pCatalog = m_vecObjects->GetObject( pCatalog->GetReference() );
+        }
+
+        if( pCatalog
+            && pCatalog->IsDictionary() 
+            && pCatalog->GetDictionary().HasKey( PdfName("Version" ) ) ) 
+        {
+            PdfObject* pVersion = pCatalog->GetDictionary().GetKey( PdfName( "Version" ) );
+            for(int i=0;i<=MAX_PDF_VERSION_STRING_INDEX;i++)
+            {
+                if(pVersion->GetName().GetName() == s_szPdfVersionNums[i])
+                {
+                    PdfError::LogMessage( eLogSeverity_Information,
+                                          "Updating version from %s to %s\n", 
+                                          s_szPdfVersionNums[static_cast<int>(m_ePdfVersion)],
+                                          s_szPdfVersionNums[i] );
+                    m_ePdfVersion = static_cast<EPdfVersion>(i);
+                    break;
+                }
+            }
+        }
+    }
+    
+}
+
 };
 
 
