@@ -25,7 +25,6 @@
 
 #include "PdfDefines.h"
 #include "PdfRefCountedBuffer.h"
-
 #include "PdfString.h"
 
 namespace PoDoFo {
@@ -72,7 +71,7 @@ class PODOFO_API PdfVariant {
     /** Construct a PdfVariant that is a number.
      *  \param l the value of the number.
      */
-    PdfVariant( long l );
+    PdfVariant( pdf_int64 l );
 
     /** Construct a PdfVariant that is a real number.
      *  \param d the value of the real number.
@@ -243,7 +242,7 @@ class PODOFO_API PdfVariant {
     /** Get the value of the object as long.
      *  \return the value of the number
      */
-    inline long GetNumber() const;
+    inline pdf_int64 GetNumber() const;
 
     /** Set the value of this object as double
      *  \param d the value as double.
@@ -291,6 +290,16 @@ class PODOFO_API PdfVariant {
      */
     inline const PdfReference & GetReference() const;
 
+    /** Get the reference values of this object.
+     *  \returns a reference to the PdfData instance.
+     */
+    inline const PdfData & GetRawData() const;
+
+    /** Get the reference values of this object.
+     *  \returns a reference to the PdfData instance.
+     */
+    inline PdfData & GetRawData();
+
     /** Assign the values of another PdfVariant to this one.
      *  \param rhs an existing variant which is copied.
      *
@@ -324,8 +333,37 @@ class PODOFO_API PdfVariant {
      */
     inline bool IsDirty() const;
 
+    /**
+     * Sets this object to immutable,
+     * so that no keys can be edited or changed.
+     *
+     * @param bImmutable if true set the object to be immutable
+     *
+     * This is used by PdfImmediateWriter and PdfStreamedDocument so 
+     * that no keys can be added to an object after setting stream data on it.
+     *
+     */
+    inline void SetImmutable(bool bImmutable);
+
+    /**
+     * Retrieve if an object is immutable.
+     *
+     * This is used by PdfImmediateWriter and PdfStreamedDocument so 
+     * that no keys can be added to an object after setting stream data on it.
+     *
+     * @returns true if the object is immutable
+     */
+    inline bool GetImmutable() const;
+
  protected:
 
+    /**
+     *  Will throw an exception if called on an immutable object,
+     *  so this should be called before actually changing a value!
+     * 
+     */
+    inline void AssertMutable() const;
+
     /** Sets the dirty flag of this PdfVariant
      *
      *  \param bDirty true if this PdfVariant has been
@@ -443,12 +481,13 @@ class PODOFO_API PdfVariant {
 
         bool       bBoolValue;
         double     dNumber;
-        long       nNumber;
+        pdf_int64      nNumber;
     } UVariant;
 
     UVariant     m_Data;
 
     bool         m_bDirty; ///< Indicates if this object was modified after construction
+    bool         m_bImmutable; ///< Indicates if this object maybe modified
 
 
     /** Datatype of the variant.
@@ -531,6 +570,7 @@ void PdfVariant::SetBool( bool b )
         PODOFO_RAISE_ERROR( ePdfError_InvalidDataType );
     }
 
+    AssertMutable();
     m_Data.bBoolValue = b;
     SetDirty( true );
 }
@@ -562,18 +602,18 @@ void PdfVariant::SetNumber( long l )
         PODOFO_RAISE_ERROR( ePdfError_InvalidDataType );
     }
 
+    AssertMutable();
     if ( IsReal() )
         m_Data.dNumber = static_cast<double>(l);
     else
         m_Data.nNumber = l;
-
     SetDirty( true );
 }
 
 // -----------------------------------------------------
 // 
 // -----------------------------------------------------
-long PdfVariant::GetNumber() const
+pdf_int64 PdfVariant::GetNumber() const
 {
     DelayedLoad();
 
@@ -600,11 +640,11 @@ void PdfVariant::SetReal( double d )
         PODOFO_RAISE_ERROR( ePdfError_InvalidDataType );
     }
 
+    AssertMutable();
     if ( IsReal() )
         m_Data.dNumber = d;
     else
         m_Data.nNumber = static_cast<long>(floor( d ));
-
     SetDirty( true );
 }
 
@@ -626,6 +666,31 @@ double PdfVariant::GetReal() const
         return static_cast<double>(m_Data.nNumber);
 }
 
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+const PdfData & PdfVariant::GetRawData() const
+{
+    DelayedLoad();
+
+    if( !IsRawData() )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InvalidDataType );
+    }
+    return *(reinterpret_cast<PdfData* const>(m_Data.pData));
+}
+
+PdfData & PdfVariant::GetRawData()
+{
+    DelayedLoad();
+
+    if( !IsRawData() )
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InvalidDataType );
+    }
+    return *(reinterpret_cast<PdfData* const>(m_Data.pData));
+}
+
 // -----------------------------------------------------
 // 
 // -----------------------------------------------------
@@ -872,6 +937,56 @@ void PdfVariant::SetDirty( bool bDirty )
     }
 }
 
+
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+inline void PdfVariant::SetImmutable(bool bImmutable)
+{
+    m_bImmutable = bImmutable;
+
+    switch( m_eDataType ) 
+    {
+        case ePdfDataType_Array:
+        case ePdfDataType_Dictionary:
+            // Arrays and Dictionaries
+            // handle dirty status by themselfes
+            m_Data.pData->SetImmutable( m_bImmutable );
+            
+        case ePdfDataType_Bool:
+        case ePdfDataType_Number:
+        case ePdfDataType_Real:
+        case ePdfDataType_HexString:
+        case ePdfDataType_String:
+        case ePdfDataType_Name:
+        case ePdfDataType_RawData:
+        case ePdfDataType_Reference:
+        case ePdfDataType_Null:
+        case ePdfDataType_Unknown:
+        default:
+            break;
+    };    
+}
+
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+inline bool PdfVariant::GetImmutable() const 
+{
+    return m_bImmutable;
+}
+
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+inline void PdfVariant::AssertMutable() const
+{
+    if(m_bImmutable) 
+    {
+        throw new PdfError( ePdfError_ChangeOnImmutable );
+    }
+}
+
 };
 
 #endif // _PDF_VARIANT_H_
